#!/usr/bin/env python
# -*- coding: ISO-8859-1 -*-
#
#
# Copyright (C) 2004 André Wobst <wobsta@users.sourceforge.net>
#
# This file is part of PyX (https://pyx-project.org/).
#
# PyX is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# PyX is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyX; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


import numpy


def sum(list):
    # we can assume len(list) != 0 here (and do not start from the scalar 0)
    sum = list[0]
    for item in list[1:]:
        sum += item
    return sum

def product(list):
    # we can assume len(list) != 0 here (and do not start from the scalar 1)
    product = list[0]
    for item in list[1:]:
        product *= item
    return product


class scalar:
    # represents a scalar variable or constant

    def __init__(self, value=None, name="unnamed_scalar"):
        self._scalar = None
        if value is not None:
            self.set(value)
        self.name = name

    def scalar(self):
        return self

    def addend(self):
        return addend([self])

    def polynom(self):
        return self.addend().polynom()

    def __neg__(self):
        return -self.addend()

    def __add__(self, other):
        return self.polynom() + other

    __radd__ = __add__

    def __sub__(self, other):
        return self.polynom() - other

    def __rsub__(self, other):
        return -self.polynom() + other

    def __mul__(self, other):
        return self.addend()*other

    __rmul__ = __mul__

    def __div__(self, other):
        return self.addend()/other

    def is_set(self):
        return self._scalar is not None

    def set(self, value):
        if self.is_set():
            raise RuntimeError("scalar already defined")
        try:
            self._scalar = float(value)
        except Exception:
            raise RuntimeError("float expected")

    def get(self):
        if not self.is_set():
            raise RuntimeError("scalar not yet defined")
        return self._scalar

    def __float__(self):
        return self.get()

    def __str__(self):
        if self.is_set():
            return "%s{=%s}" % (self.name, self._scalar)
        else:
            return self.name


class addend:
    # represents a addend, i.e. list of scalars to be multiplied by each other

    def __init__(self, scalars):
        self._scalars = [scalar.scalar() for scalar in scalars]
        if not len(self._scalars):
            raise RuntimeError("empty scalars not allowed")

    def addend(self):
        return self

    def polynom(self):
        return polynom([self])

    def __neg__(self):
        return addend([scalar(-1)] + self._scalars)

    def __add__(self, other):
        return self.polynom() + other

    __radd__ = __add__

    def __sub__(self, other):
        return self.polynom() - other

    def __rsub__(self, other):
        return -self.polynom() + other

    def __mul__(self, other):
        try:
            other = other.addend()
        except (TypeError, AttributeError):
            try:
                other = scalar(other)
            except RuntimeError:
                return other * self
            else:
                return addend(self._scalars + [other])
        else:
            return addend(self._scalars + other._scalars)

    __rmul__ = __mul__

    def __div__(self, other):
        return addend([scalar(1/other)] + self._scalars)

    def __float__(self):
        return product([float(scalar) for scalar in self._scalars])

    def is_linear(self):
        return len([scalar for scalar in self._scalars if not scalar.is_set()]) < 2

    def prefactor(self):
        assert self.is_linear()
        setscalars = [scalar for scalar in self._scalars if scalar.is_set()]
        if len(setscalars):
            return float(addend(setscalars))
        else:
            return 1

    def variable(self):
        assert self.is_linear()
        unsetscalars = [scalar for scalar in self._scalars if not scalar.is_set()]
        if len(unsetscalars):
            assert len(unsetscalars) == 1
            return unsetscalars[0]
        else:
            return None

    def __str__(self):
        return " * ".join([str(scalar) for scalar in self._scalars])


class polynom:
    # represents a polynom, i.e. a list of addends to be summed up

    def __init__(self, polynom):
        self._addends = [addend.addend() for addend in polynom]
        if not len(self._addends):
            raise RuntimeError("empty polynom not allowed")

    def polynom(self):
        return self

    def __neg__(self):
        return polynom([-addend for addend in self._addends])

    def __add__(self, other):
        try:
            other = other.polynom()
        except (TypeError, AttributeError):
            other = scalar(other).polynom()
        return polynom(self._addends + other._addends)

    __radd__ = __add__

    def __sub__(self, other):
        return -other + self

    def __rsub__(self, other):
        return -self + other

    def __mul__(self, other):
        return sum([addend*other for addend in self._addends])

    __rmul__ = __mul__

    def __div__(self, other):
        return polynom([addend/other for addend in self._addends])

    def __float__(self):
        return sum([float(addend) for addend in self._addends])

    def is_linear(self):
        is_linear = 1
        for addend in self._addends:
            is_linear = is_linear and addend.is_linear()
        return is_linear

    def __str__(self):
        return "  +  ".join([str(addend) for addend in self._addends])

    def solve(self, solver):
        solver.addequation(self)


class vector:
    # represents a vector, i.e. a list of scalars (or polynoms)

    def __init__(self, dimension_or_values, name="unnamed_vector"):
        try:
            name + ""
        except Exception:
            raise RuntimeError("a vectors name should be a string (you probably wanted to write vector([x, y]) instead of vector(x, y))")
        try:
            for value in dimension_or_values:
                pass
        except Exception:
            # dimension
            self._items = [scalar(name="%s[%i]" % (name, i))
                           for i in range(dimension_or_values)]
        else:
            # values
            self._items = []
            for value in dimension_or_values:
                try:
                    value.polynom()
                except (TypeError, AttributeError):
                    self._items.append(scalar(value=value, name="%s[%i]" % (name, len(self._items))))
                else:
                    self._items.append(value)
        if not len(self._items):
            raise RuntimeError("empty vector not allowed")
        self.name = name

    def __len__(self):
        return len(self._items)

    def __getitem__(self, i):
        return self._items[i]

    def __getattr__(self, attr):
        if attr == "x":
            return self[0]
        if attr == "y":
            return self[1]
        if attr == "z":
            return self[2]
        else:
            raise AttributeError(attr)

    def vector(self):
        return self

    def __neg__(self):
        return vector([-item for item in self._items])

    def __add__(self, other):
        other = other.vector()
        if len(self) != len(other):
            raise RuntimeError("vector length mismatch in add")
        return vector([selfitem + otheritem for selfitem, otheritem in zip(self._items, other._items)])

    __radd__ = __add__

    def __sub__(self, other):
        return -other + self

    def __rsub__(self, other):
        return -self + other

    def __mul__(self, other):
        try:
            other = other.vector()
        except (TypeError, AttributeError):
            return vector([item*other for item in self._items])
        else:
            # scalar product
            if len(self) != len(other):
                raise RuntimeError("vector length mismatch in scalar product")
            return sum([selfitem*otheritem for selfitem, otheritem in zip(self._items, other._items)])

    def __rmul__(self, other):
        return vector([other*item for item in self._items])
        # We do not allow for vector * <matrix-like-object> here (i.e. a
        # simulating the behaviour of a dual vector). In principle we could do
        # that, but for transformations it would lead to (a*X)*c != a*(X*c).
        # Hence we disallow vector*X for X being something else that a scalar.

    def __div__(self, other):
        return vector([item/other for item in self._items])

    def __str__(self):
        return "%s{=(%s)}" % (self.name, ", ".join([str(item) for item in self._items]))

    def solve(self, solver):
        for item in self._items:
            solver.addequation(item)


class zerovector(vector):

    def __init__(self, dimension, name="0"):
        vector.__init__(self, [0 for i in range(dimension)], name)


class matrix:
    # represents a matrix, i.e. a 2d list of scalars (or polynoms)

    def __init__(self, dimensions_or_values, name="unnamed_matrix"):
        try:
            name + ""
        except Exception:
            raise RuntimeError("a matrix name should be a string (you probably wanted to write matrix([x, y]) instead of matrix(x, y))")
        try:
            for row in dimensions_or_values:
                for col in row:
                    pass
        except Exception:
            # dimension
            self._numberofrows, self._numberofcols = [int(x) for x in dimensions_or_values]
            self._rows = [[scalar(name="%s[%i, %i]" % (name, row, col))
                           for col in range(self._numberofcols)]
                          for row in range(self._numberofrows)]
        else:
            # values
            self._rows = []
            self._numberofcols = None
            for row in dimensions_or_values:
                _cols = []
                for col in row:
                    try:
                        col.polynom()
                    except (TypeError, AttributeError):
                        _cols.append(scalar(value=col, name="%s[%i, %i]" % (name, len(self._rows), len(_cols))))
                    else:
                        _cols.append(col)
                self._rows.append(_cols)
                if self._numberofcols is None:
                    self._numberofcols = len(_cols)
                elif self._numberofcols != len(_cols):
                    raise RuntimeError("column length mismatch")
            self._numberofrows = len(self._rows)
        if not self._numberofrows or not self._numberofcols:
            raise RuntimeError("empty matrix not allowed")
        self.name = name

    # instead of __len__ two methods to fetch the matrix dimensions
    def getnumberofrows(self):
        return self._numberofrows

    def getnumberofcols(self):
        return self._numberofcols

    def __getitem__(self, xxx_todo_changeme):
        (row, col) = xxx_todo_changeme
        return self._rows[row][col]

    def matrix(self):
        return self

    def __neg__(self):
        return matrix([[-col for col in row] for row in self._rows])

    def __add__(self, other):
        other = other.matrix()
        if self._numberofrows != other._numberofrows or self._numberofcols != other._numberofcols:
            raise RuntimeError("matrix geometry mismatch in add")
        return matrix([[selfcol + othercol
                        for selfcol, othercol in zip(selfrow, otherrow)]
                       for selfrow, otherrow in zip(self._rows, other._rows)])

    __radd__ = __add__

    def __sub__(self, other):
        return -other + self

    def __rsub__(self, other):
        return -self + other

    def __mul__(self, other):
        try:
            other = other.matrix()
        except (TypeError, AttributeError):
            try:
                other = other.vector()
            except (TypeError, AttributeError):
                return matrix([[col*other for col in row] for row in self._rows])
            else:
                if self._numberofcols != len(other):
                    raise RuntimeError("size mismatch in matrix vector product")
                return vector([sum([col*otheritem
                                    for col, otheritem in zip(row, other)])
                               for row in self._rows])
        else:
            if self._numberofcols != other._numberofrows:
                raise RuntimeError("size mismatch in matrix product")
            return matrix([[sum([self._rows[row][i]*other._rows[i][col]
                                 for i in range(self._numberofcols)])
                            for col in range(other._numberofcols)]
                           for row in range(self._numberofrows)])

    def __rmul__(self, other):
        try:
            other = other.vector()
        except (TypeError, AttributeError):
            return matrix([[other*col for col in row] for row in self._rows])
        else:
            if self._numberofrows != len(other):
                raise RuntimeError("size mismatch in matrix vector product")
            return vector([sum([other[i]*self._rows[i][col]
                                for i in range(self._numberofrows)])
                           for col in range(self._numberofcols)])

    def __div__(self, other):
        return matrix([[col/other for col in row] for row in self._rows])

    def __str__(self):
        return "%s{=(%s)}" % (self.name, ", ".join(["(" + ", ".join([str(col) for col in row]) + ")" for row in self._rows]))

    def solve(self, solver):
        for row in self._rows:
            for col in row:
                solver.addequation(col)


class identitymatrix(matrix):

    def __init__(self, dimension, name="I"):
        def eq(row, col):
            if row == col:
                return 1
            else:
                return 0
        matrix.__init__(self, [[eq(row, col) for col in range(dimension)] for row in range(dimension)], name)


class trafo:
    # represents a transformation, i.e. matrix and a constant vector

    def __init__(self, dimensions_or_values, name="unnamed_trafo"):
        try:
            name + ""
        except Exception:
            raise RuntimeError("a trafo name should be a string (you probably wanted to write trafo([x, y]) instead of trafo(x, y))")
        if len(dimensions_or_values) != 2:
            raise RuntimeError("first parameter of a trafo must contain two elements: either two dimensions or a matrix and a vector")
        try:
            numberofrows, numberofcols = [int(x) for x in dimensions_or_values]
        except Exception:
            self._matrix = dimensions_or_values[0].matrix()
            self._vector = dimensions_or_values[1].vector()
            if self._matrix.getnumberofrows() != len(self._vector):
                raise RuntimeError("size mismatch between matrix and vector")
        else:
            self._matrix = matrix((numberofrows, numberofcols), name=name + "_matrix")
            self._vector = vector(numberofrows, name=name + "_vector")
        self.name = name

    def trafo(self):
        return self

    def getmatrix(self):
        return self._matrix

    def getvector(self):
        return self._vector

    def __neg__(self):
        return trafo((-self._matrix, -self._vector))

    def __add__(self, other):
        other = other.trafo()
        return trafo((self._matrix + other._matrix, self._vector + other._vector))

    __radd__ = __add__

    def __sub__(self, other):
        return -other + self

    def __rsub__(self, other):
        return -self + other

    def __mul__(self, other):
        try:
            other = other.trafo()
        except (TypeError, AttributeError):
            try:
                other = other.vector()
            except (TypeError, AttributeError):
                return trafo((self._matrix * other, self._vector * other))
            else:
                return self._matrix * other + self._vector
        else:
            return trafo((self._matrix * other._matrix, self._vector + self._matrix * other._vector))

    def __rmul__(self, other):
        return trafo((other * self._matrix, other * self._vector))

    def __div__(self, other):
        return trafo((self._matrix/other, self._vector/other))

    def __str__(self):
        return "%s{=(matrix: %s, vector: %s)}" % (self.name, self._matrix, self._vector)

    def solve(self, solver):
        self._matrix.solve(solver)
        self._vector.solve(solver)


class identitytrafo(trafo):

    def __init__(self, dimension, name="I"):
        trafo.__init__(self, (identitymatrix(dimension, name=name),
                              zerovector(dimension, name=name)), name)


class Solver:
    # linear equation solver

    def __init__(self):
        self.eqs = [] # scalar equations not yet solved (a equation is a polynom to be zero)

    def eq(self, lhs, rhs=None):
        if rhs is None:
            eq = lhs
        else:
            eq = lhs - rhs
        eq.solve(self)

    def addequation(self, equation):
        # the equation is just a polynom which should be zero
        self.eqs.append(equation.polynom())

        # try to solve some combinations of linear equations
        while 1:
            for eqs in self.combine(self.eqs):
                if self.solve(eqs):
                    break # restart for loop
            else:
                break # quit while loop

    def combine(self, eqs):
        # create combinations of linear equations
        if not len(eqs):
            yield []
        else:
            for x in self.combine(eqs[1:]):
                yield x
            if eqs[0].is_linear():
                for x in self.combine(eqs[1:]):
                    yield [eqs[0]] + x

    def solve(self, eqs):
        # try to solve a set of linear equations
        l = len(eqs)
        if l:
            vars = []
            for eq in eqs:
                for addend in eq._addends:
                    var = addend.variable()
                    if var is not None and var not in vars:
                        vars.append(var)
            if len(vars) == l:
                a = numpy.zeros((l, l), numpy.float)
                b = numpy.zeros((l, ), numpy.float)
                for i, eq in enumerate(eqs):
                    for addend in eq._addends:
                        var = addend.variable()
                        if var is not None:
                            a[i, vars.index(var)] += addend.prefactor()
                        else:
                            b[i] -= addend.prefactor()
                for i, value in enumerate(numpy.linalg.solve(a, b)):
                    vars[i].set(value)
                for eq in eqs:
                    i, = [i for i, selfeq in enumerate(self.eqs) if selfeq == eq]
                    del self.eqs[i]
                return 1
            elif len(vars) < l:
                raise RuntimeError("equations are overdetermined")
        return 0

solver = Solver()





import sys; sys.path[:0] = ["../.."]
import random, string
from pyx import *

text.set(texipc=1)
text.set(mode="tex", texdebug="debug.tex", usefiles=["debug.dvi", "debug.log"])

def randpar():
    return " ".join(["".join([random.choice(string.lowercase)
                              for j in range(random.randint(1, 3))])
                     for i in range(random.randint(100, 300))])

def randparwithdisplay():
    #par = "" # TODO allow for formulas at the beginning -- to be checked
    par = randpar()
    for i in range(random.randint(0, 3)):
        par += r"\display{}xxx\enddisplay{}"
        for j in range(random.randint(0, 1)):
            par += randpar()
    #return par # TODO allow for formulas at the end -- to be checked
    return par + randpar()

def randtext():
    return r"\par{}".join([randparwithdisplay() for i in range(random.randint(1, 10))])

def output(boxes, shapes):
    c = canvas.canvas()
    y = 0
    tr = text.texrunner()
    for i in range(len(boxes)):
        if i < len(shapes):
            shape = shapes[i]
        c.stroke(path.rect(0, y, shape[0], -shape[1]))
        c.stroke(boxes[i].bbox().path(), [trafo.translate(0, y), color.rgb.red])
        c.insert(boxes[i], [trafo.translate(0, y)])
        for mark in list(boxes[i].markers.keys()):
            mx, my = boxes[i].markers[mark]
            c.insert(tr.text(mx,my+y, mark+"~", [text.size.tiny, text.halign.right]))
            if mark[:5] == "start":
                c.fill(path.circle(mx, my+y, 0.05), [color.rgb.red])
            elif mark[:3] == "end":
                c.fill(path.circle(mx, my+y, 0.05), [color.rgb.green])
            else:
                raise ValueError("other marks in there!")
        y -= shape[1] + 3
    c.writeEPSfile("textboxes")

random.seed(0)
shapes = [(10,7), (8,5)]*50
n = 0
only = 28
while only is None or n <= only:
    print(n)
    n += 1
    if 1:
        thistext = randtext()
        thistextfile = open("debug.thistext", "w")
        thistextfile.write(thistext)
        thistextfile.close()
    else:
        thistextfile = open("debug.thistext", "r")
        thistext = thistextfile.read()
        thistextfile.close()
    if only is not None and n <= only:
        continue
    boxes = text.defaulttexrunner.textboxes(thistext, shapes)
    output(boxes, shapes)
    for i in range(len(boxes)):
        if i < len(shapes):
            shape = shapes[i]
        if abs(unit.topt(boxes[i].bbox().right()) - unit.topt(shape[0])) > 1:
            print(("right boundary differs:",
                   unit.topt(boxes[i].bbox().bottom()), -unit.topt(shape[1]),
                   unit.topt(boxes[i].bbox().right()), unit.topt(shape[0]),
                   i+1, len(boxes)))




#!/usr/bin/env python
import sys; sys.path[:0] = ["../.."]

from pyx import *

c = canvas.canvas()

pa1= pattern.pattern(attrs=[style.linewidth.thin])
pa1.stroke(path.line(0, 0, 0.25, 0.25))
pa1.stroke(path.line(0, 0.25, 0.25, 0))

#c.draw(path.circle(3, 3, 3), [deco.filled([pa1])])
c.draw(path.circle(3, 3, 3), [deco.filled([pa1]), deco.stroked])
c.draw(path.circle(5, 1, 1), [deco.filled([pa1]), deco.stroked])

pa2 = pattern.pattern()
pa2.text(0.125, 0.125, r"\PyX")

c.draw(path.rect(-1, -1, 1, 1), [deco.filled([pa2]), deco.stroked])

c.writeEPSfile("test_pattern", page_paperformat=document.paperformat.A4)
c.writePDFfile("test_pattern", page_paperformat=document.paperformat.A4)
c.writeSVGfile("test_pattern", page_paperformat=document.paperformat.A4)



import unittest, operator

from solve import scalar, vector, matrix, identitymatrix, trafo, identitytrafo, Solver

class ScalarTestCase(unittest.TestCase):

    def testInit(self):
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), scalar, "")
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), scalar, 1j)
        self.failUnlessEqual(str(scalar()), "unnamed_scalar")
        self.failUnlessEqual(str(scalar(name="s")), "s")
        self.failUnlessEqual(str(scalar(1)), "unnamed_scalar{=1.0}")
        self.failUnlessEqual(str(scalar(-1, name="s")), "s{=-1.0}")

    def testMath(self):
        self.failUnlessEqual(str(-scalar(name="s")), "unnamed_scalar{=-1.0} * s")
        self.failUnlessEqual(str(scalar(name="s") + scalar(name="t")), "s  +  t")
        self.failUnlessEqual(str(scalar(name="s") + 1), "s  +  unnamed_scalar{=1.0}")
        self.failUnlessEqual(str(1 + scalar(name="s")), "s  +  unnamed_scalar{=1.0}")
        self.failUnlessEqual(str(scalar(name="s") - scalar(name="t")), "unnamed_scalar{=-1.0} * t  +  s")
        self.failUnlessEqual(str(1 - scalar(name="s")), "unnamed_scalar{=-1.0} * s  +  unnamed_scalar{=1.0}")
        self.failUnlessEqual(str(2 * scalar(name="s")), "s * unnamed_scalar{=2.0}")
        self.failUnlessEqual(str(scalar(name="s") * 2), "s * unnamed_scalar{=2.0}")
        self.failUnlessEqual(str(scalar(name="s") * scalar(name="t")), "s * t")
        self.failUnlessEqual(str((scalar(name="s") + scalar(name="t")) * 2), "s * unnamed_scalar{=2.0}  +  t * unnamed_scalar{=2.0}")
        self.failUnlessEqual(str(scalar(name="s") / 2.0), "unnamed_scalar{=0.5} * s")
        self.failUnlessEqual(str(scalar(name="s") / 2), "unnamed_scalar{=0.0} * s") # integer logic!
        self.failUnlessEqual(str((scalar(name="s") + scalar(name="t")) / 2.0), "unnamed_scalar{=0.5} * s  +  unnamed_scalar{=0.5} * t")
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__div__, 2, scalar())
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__div__, scalar(), scalar())
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__div__, vector(1), scalar())
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__div__, scalar() + scalar(), scalar())
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__div__, vector(1) + vector(1), scalar())

    def testAccess(self):
        s = scalar()
        self.failUnlessEqual(s.is_set(), 0)
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), s.get)
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), float, s)
        s.set(2)
        self.failUnlessEqual(s.is_set(), 1)
        self.failUnlessAlmostEqual(s.get(), 2.0)
        self.failUnlessAlmostEqual(float(s), 2.0)
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), s.set, 3)
        self.failUnlessEqual(s.is_set(), 1)
        self.failUnlessAlmostEqual(s.get(), 2.0)
        self.failUnlessAlmostEqual(float(s), 2.0)

        s = scalar(2)
        self.failUnlessEqual(scalar(2).is_set(), 1)
        self.failUnlessAlmostEqual(s.get(), 2.0)
        self.failUnlessAlmostEqual(float(s), 2.0)
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), s.set, 3)
        self.failUnlessEqual(s.is_set(), 1)
        self.failUnlessAlmostEqual(s.get(), 2.0)
        self.failUnlessAlmostEqual(float(s), 2.0)


class VectorTestCase(unittest.TestCase):

    def testInit(self):
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), vector, 0, 0)
        self.failUnlessEqual(str(vector(2)), "unnamed_vector{=(unnamed_vector[0], unnamed_vector[1])}")
        self.failUnlessEqual(str(vector([1, 2])), "unnamed_vector{=(unnamed_vector[0]{=1.0}, unnamed_vector[1]{=2.0})}")
        self.failUnlessEqual(str(vector(3, "a")), "a{=(a[0], a[1], a[2])}")
        self.failUnlessEqual(str(vector([3, 2, 1], "a")), "a{=(a[0]{=3.0}, a[1]{=2.0}, a[2]{=1.0})}")

    def testAccess(self):
        a = vector(2)
        self.failUnlessEqual(str(a), "unnamed_vector{=(unnamed_vector[0], unnamed_vector[1])}")
        self.failUnlessEqual(a[0].is_set(), 0)
        self.failUnlessEqual(a.x.is_set(), 0)
        self.failUnlessEqual(a[1].is_set(), 0)
        self.failUnlessEqual(a.y.is_set(), 0)
        self.failUnlessRaises(IndexError, operator.__getitem__, a, 2)
        self.failUnlessRaises(IndexError, getattr, a, "z")

        a[0].set(2)
        self.failUnlessEqual(str(a), "unnamed_vector{=(unnamed_vector[0]{=2.0}, unnamed_vector[1])}")
        self.failUnlessAlmostEqual(a[0].get(), 2.0)
        self.failUnlessAlmostEqual(float(a[0]), 2.0)
        self.failUnlessAlmostEqual(a.x.get(), 2.0)
        self.failUnlessAlmostEqual(float(a.x), 2.0)
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), a[1].get)
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), float, a[1])
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), a.y.get)
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), float, a.y)
        self.failUnlessEqual(a[0].is_set(), 1)
        self.failUnlessEqual(a.x.is_set(), 1)
        self.failUnlessEqual(a[1].is_set(), 0)
        self.failUnlessEqual(a.y.is_set(), 0)
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), a[0].set, 3)
        self.failUnlessAlmostEqual(a[0].get(), 2.0)
        self.failUnlessAlmostEqual(float(a[0]), 2.0)
        self.failUnlessAlmostEqual(a.x.get(), 2.0)
        self.failUnlessAlmostEqual(float(a.x), 2.0)
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), a[1].get)
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), float, a[1])
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), a.y.get)
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), float, a.y)
        self.failUnlessEqual(a[0].is_set(), 1)
        self.failUnlessEqual(a.x.is_set(), 1)
        self.failUnlessEqual(a[1].is_set(), 0)
        self.failUnlessEqual(a.y.is_set(), 0)

        a[1].set(3)
        self.failUnlessEqual(str(a), "unnamed_vector{=(unnamed_vector[0]{=2.0}, unnamed_vector[1]{=3.0})}")
        self.failUnlessAlmostEqual(a[0].get(), 2.0)
        self.failUnlessAlmostEqual(float(a[0]), 2.0)
        self.failUnlessAlmostEqual(a.x.get(), 2.0)
        self.failUnlessAlmostEqual(float(a.x), 2.0)
        self.failUnlessAlmostEqual(a[1].get(), 3.0)
        self.failUnlessAlmostEqual(float(a[1]), 3.0)
        self.failUnlessAlmostEqual(a.y.get(), 3.0)
        self.failUnlessAlmostEqual(float(a.y), 3.0)
        self.failUnlessEqual(a[0].is_set(), 1)
        self.failUnlessEqual(a.x.is_set(), 1)
        self.failUnlessEqual(a[1].is_set(), 1)
        self.failUnlessEqual(a.y.is_set(), 1)
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), a[0].set, 4)
        self.failUnlessAlmostEqual(a[0].get(), 2.0)
        self.failUnlessAlmostEqual(float(a[0]), 2.0)
        self.failUnlessAlmostEqual(a.x.get(), 2.0)
        self.failUnlessAlmostEqual(float(a.x), 2.0)
        self.failUnlessAlmostEqual(a[1].get(), 3.0)
        self.failUnlessAlmostEqual(float(a[1]), 3.0)
        self.failUnlessAlmostEqual(a.y.get(), 3.0)
        self.failUnlessAlmostEqual(float(a.y), 3.0)
        self.failUnlessEqual(a[0].is_set(), 1)
        self.failUnlessEqual(a.x.is_set(), 1)
        self.failUnlessEqual(a[1].is_set(), 1)
        self.failUnlessEqual(a.y.is_set(), 1)

        a = vector([1, 2, 3])
        self.failUnlessEqual(str(a.x), "unnamed_vector[0]{=1.0}")
        self.failUnlessEqual(str(a.y), "unnamed_vector[1]{=2.0}")
        self.failUnlessEqual(str(a.z), "unnamed_vector[2]{=3.0}")

    def testLen(self):
        for i in range(1, 5):
            a = vector(i)
            self.failUnlessEqual(len(a), i)
            self.failUnlessEqual(str(a), "unnamed_vector{=(" + ", ".join(["unnamed_vector[%i]" % j for j in range(i)]) + ")}")

    def testMath(self):
        self.failUnlessEqual(str(-vector(2, "a")), "unnamed_vector{=(unnamed_scalar{=-1.0} * a[0], unnamed_scalar{=-1.0} * a[1])}")
        self.failUnlessEqual(str(vector(2, "a") + vector(2, "b")), "unnamed_vector{=(a[0]  +  b[0], a[1]  +  b[1])}")
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__add__, 1, vector(2))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__add__, vector(2), 1)
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__add__, scalar(), vector(2))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__add__, vector(2), scalar())
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__add__, scalar() + scalar(), vector(2))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__add__, vector(2), scalar() + scalar())
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__add__, vector(2), vector(3))
        self.failUnlessEqual(str(vector(2, "a") - vector(2, "b")), "unnamed_vector{=(unnamed_scalar{=-1.0} * b[0]  +  a[0], unnamed_scalar{=-1.0} * b[1]  +  a[1])}")
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__sub__, 1, vector(2))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__sub__, vector(2), 1)
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__sub__, scalar(), vector(2))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__sub__, vector(2), scalar())
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__sub__, scalar() + scalar(), vector(2))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__sub__, vector(2), scalar() + scalar())
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__sub__, vector(2), vector(3))
        self.failUnlessEqual(str(2 * vector(2, "a")), "unnamed_vector{=(a[0] * unnamed_scalar{=2.0}, a[1] * unnamed_scalar{=2.0})}")
        self.failUnlessEqual(str(vector(2, "a") * 2), "unnamed_vector{=(a[0] * unnamed_scalar{=2.0}, a[1] * unnamed_scalar{=2.0})}")
        self.failUnlessEqual(str(scalar(name="s") * vector(2, "a")), "unnamed_vector{=(a[0] * s, a[1] * s)}")
        self.failUnlessEqual(str(scalar(name="s") * (vector(2, "a") + vector(2, "b"))), "unnamed_vector{=(a[0] * s  +  b[0] * s, a[1] * s  +  b[1] * s)}")
        self.failUnlessEqual(str((scalar(name="s") + scalar(name="t")) * vector(2, "a")), "unnamed_vector{=(a[0] * s  +  a[0] * t, a[1] * s  +  a[1] * t)}")
        self.failUnlessEqual(str((scalar(name="s") + scalar(name="t")) * (vector(2, "a") + vector(2, "b"))), "unnamed_vector{=(a[0] * s  +  b[0] * s  +  a[0] * t  +  b[0] * t, a[1] * s  +  b[1] * s  +  a[1] * t  +  b[1] * t)}")
        self.failUnlessEqual(str(vector(2, "a") * scalar(name="s")), "unnamed_vector{=(a[0] * s, a[1] * s)}")
        self.failUnlessEqual(str(vector(2, "a") * vector(2, "b")), "a[0] * b[0]  +  a[1] * b[1]")
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__mul__, vector(2, "a"), vector(3))
        self.failUnlessEqual(str(vector(2, "a") / 2.0), "unnamed_vector{=(unnamed_scalar{=0.5} * a[0], unnamed_scalar{=0.5} * a[1])}")
        self.failUnlessEqual(str(vector(2, "a") / 2), "unnamed_vector{=(unnamed_scalar{=0.0} * a[0], unnamed_scalar{=0.0} * a[1])}") # integer logic!
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__div__, scalar(), vector(1))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__div__, scalar() + scalar(), vector(1))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__div__, vector(1), scalar())
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__div__, vector(1), scalar() + scalar())
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__div__, vector(1), vector(1))


class MatrixTestCase(unittest.TestCase):

    def testInit(self):
        self.failUnlessEqual(str(matrix((2, 3))), "unnamed_matrix{=((unnamed_matrix[0, 0], unnamed_matrix[0, 1], unnamed_matrix[0, 2]), (unnamed_matrix[1, 0], unnamed_matrix[1, 1], unnamed_matrix[1, 2]))}")
        self.failUnlessEqual(str(matrix([[1, 2, 3], [4, 5, 6]])), "unnamed_matrix{=((unnamed_matrix[0, 0]{=1.0}, unnamed_matrix[0, 1]{=2.0}, unnamed_matrix[0, 2]{=3.0}), (unnamed_matrix[1, 0]{=4.0}, unnamed_matrix[1, 1]{=5.0}, unnamed_matrix[1, 2]{=6.0}))}")
        self.failUnlessEqual(str(matrix((2, 3), "a")), "a{=((a[0, 0], a[0, 1], a[0, 2]), (a[1, 0], a[1, 1], a[1, 2]))}")
        self.failUnlessEqual(str(matrix([[1, 2, 3], [4, 5, 6]], "a")), "a{=((a[0, 0]{=1.0}, a[0, 1]{=2.0}, a[0, 2]{=3.0}), (a[1, 0]{=4.0}, a[1, 1]{=5.0}, a[1, 2]{=6.0}))}")
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), matrix, [[1, 2, 3], [4, 5]])

    def testLen(self):
        for rows in range(1, 5):
            for cols in range(1, 5):
                A = matrix((rows, cols))
                self.failUnlessEqual(A.getnumberofrows(), rows)
                self.failUnlessEqual(A.getnumberofcols(), cols)
                self.failUnlessEqual(str(A), "unnamed_matrix{=(" + ", ".join(["(" + ", ".join(["unnamed_matrix[%i, %i]" % (i, j) for j in range(cols)]) + ")" for i in range(rows)]) + ")}")

    def testAccess(self):
        A = matrix([2, 3])
        self.failUnlessEqual(str(A), "unnamed_matrix{=((unnamed_matrix[0, 0], unnamed_matrix[0, 1], unnamed_matrix[0, 2]), (unnamed_matrix[1, 0], unnamed_matrix[1, 1], unnamed_matrix[1, 2]))}")
        self.failUnlessEqual(A[0, 0].is_set(), 0)
        self.failUnlessEqual(A[1, 0].is_set(), 0)
        self.failUnlessRaises(IndexError, operator.__getitem__, A, (2, 0))
        self.failUnlessEqual(A[0, 1].is_set(), 0)
        self.failUnlessEqual(A[1, 1].is_set(), 0)
        self.failUnlessEqual(A[0, 2].is_set(), 0)
        self.failUnlessEqual(A[1, 2].is_set(), 0)
        self.failUnlessRaises(IndexError, operator.__getitem__, A, (0, 3))
        A[0, 0].set(1)
        self.failUnlessEqual(A[0, 0].is_set(), 1)
        self.failUnlessEqual(A[0, 1].is_set(), 0)
        self.failUnlessEqual(A[1, 0].is_set(), 0)
        self.failUnlessAlmostEqual(A[0, 0].get(), 1.0)
        self.failUnlessAlmostEqual(float(A[0, 0]), 1.0)

    def testMath(self):
        self.failUnlessEqual(str(-matrix([2, 3], "A")), "unnamed_matrix{=((unnamed_scalar{=-1.0} * A[0, 0], unnamed_scalar{=-1.0} * A[0, 1], unnamed_scalar{=-1.0} * A[0, 2]), (unnamed_scalar{=-1.0} * A[1, 0], unnamed_scalar{=-1.0} * A[1, 1], unnamed_scalar{=-1.0} * A[1, 2]))}")
        self.failUnlessEqual(str(matrix([2, 3], "A") + matrix([2, 3], "B")), "unnamed_matrix{=((A[0, 0]  +  B[0, 0], A[0, 1]  +  B[0, 1], A[0, 2]  +  B[0, 2]), (A[1, 0]  +  B[1, 0], A[1, 1]  +  B[1, 1], A[1, 2]  +  B[1, 2]))}")
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__add__, matrix([2, 3]), 1)
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__add__, matrix([2, 3]), scalar())
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__add__, matrix([2, 3]), scalar() + scalar())
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__add__, matrix([2, 3]), vector(2))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__add__, matrix([2, 3]), vector(3))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__add__, matrix([2, 3]), matrix([2, 4]))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__add__, matrix([2, 3]), matrix([3, 3]))
        self.failUnlessEqual(str(matrix([2, 3], "A") - matrix([2, 3], "B")), "unnamed_matrix{=((unnamed_scalar{=-1.0} * B[0, 0]  +  A[0, 0], unnamed_scalar{=-1.0} * B[0, 1]  +  A[0, 1], unnamed_scalar{=-1.0} * B[0, 2]  +  A[0, 2]), (unnamed_scalar{=-1.0} * B[1, 0]  +  A[1, 0], unnamed_scalar{=-1.0} * B[1, 1]  +  A[1, 1], unnamed_scalar{=-1.0} * B[1, 2]  +  A[1, 2]))}")
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__sub__, matrix([2, 3]), 1)
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__sub__, matrix([2, 3]), scalar())
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__sub__, matrix([2, 3]), scalar() + scalar())
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__sub__, matrix([2, 3]), vector(2))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__sub__, matrix([2, 3]), vector(3))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__sub__, matrix([2, 3]), matrix([2, 4]))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__sub__, matrix([2, 3]), matrix([3, 3]))
        self.failUnlessEqual(str(2 * matrix([2, 3], "A")), "unnamed_matrix{=((A[0, 0] * unnamed_scalar{=2.0}, A[0, 1] * unnamed_scalar{=2.0}, A[0, 2] * unnamed_scalar{=2.0}), (A[1, 0] * unnamed_scalar{=2.0}, A[1, 1] * unnamed_scalar{=2.0}, A[1, 2] * unnamed_scalar{=2.0}))}")
        self.failUnlessEqual(str(matrix([2, 3], "A") * 2), "unnamed_matrix{=((A[0, 0] * unnamed_scalar{=2.0}, A[0, 1] * unnamed_scalar{=2.0}, A[0, 2] * unnamed_scalar{=2.0}), (A[1, 0] * unnamed_scalar{=2.0}, A[1, 1] * unnamed_scalar{=2.0}, A[1, 2] * unnamed_scalar{=2.0}))}")
        self.failUnlessEqual(str(matrix([2, 3], "A") * vector(3, "a")), "unnamed_vector{=(A[0, 0] * a[0]  +  A[0, 1] * a[1]  +  A[0, 2] * a[2], A[1, 0] * a[0]  +  A[1, 1] * a[1]  +  A[1, 2] * a[2])}")
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__mul__, vector(2, "a"), matrix([2, 3], "A"))
        self.failUnlessEqual(str(matrix([2, 3], "A") * matrix([3, 2], "B")), "unnamed_matrix{=((A[0, 0] * B[0, 0]  +  A[0, 1] * B[1, 0]  +  A[0, 2] * B[2, 0], A[0, 0] * B[0, 1]  +  A[0, 1] * B[1, 1]  +  A[0, 2] * B[2, 1]), (A[1, 0] * B[0, 0]  +  A[1, 1] * B[1, 0]  +  A[1, 2] * B[2, 0], A[1, 0] * B[0, 1]  +  A[1, 1] * B[1, 1]  +  A[1, 2] * B[2, 1]))}")
        self.failUnlessEqual(str(matrix([2, 3], "A") / 2.0), "unnamed_matrix{=((unnamed_scalar{=0.5} * A[0, 0], unnamed_scalar{=0.5} * A[0, 1], unnamed_scalar{=0.5} * A[0, 2]), (unnamed_scalar{=0.5} * A[1, 0], unnamed_scalar{=0.5} * A[1, 1], unnamed_scalar{=0.5} * A[1, 2]))}")
        self.failUnlessEqual(str(matrix([2, 3], "A") / 2), "unnamed_matrix{=((unnamed_scalar{=0.0} * A[0, 0], unnamed_scalar{=0.0} * A[0, 1], unnamed_scalar{=0.0} * A[0, 2]), (unnamed_scalar{=0.0} * A[1, 0], unnamed_scalar{=0.0} * A[1, 1], unnamed_scalar{=0.0} * A[1, 2]))}")
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__div__, scalar(), matrix([2, 3]))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__div__, scalar() + scalar(), matrix([2, 3]))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__div__, vector(1), matrix([2, 3]))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__div__, matrix([2, 3]), scalar())
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__div__, matrix([2, 3]), scalar() + scalar())
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__div__, matrix([2, 3]), vector(1))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__div__, matrix([2, 3]), matrix([2, 3]))


class TrafoTestCase(unittest.TestCase):

    def testInit(self):
        self.failUnlessEqual(str(trafo((2, 3))), "unnamed_trafo{=(matrix: unnamed_trafo_matrix{=((unnamed_trafo_matrix[0, 0], unnamed_trafo_matrix[0, 1], unnamed_trafo_matrix[0, 2]), (unnamed_trafo_matrix[1, 0], unnamed_trafo_matrix[1, 1], unnamed_trafo_matrix[1, 2]))}, vector: unnamed_trafo_vector{=(unnamed_trafo_vector[0], unnamed_trafo_vector[1])})}")
        self.failUnlessEqual(str(trafo((2, 3), "A")), "A{=(matrix: A_matrix{=((A_matrix[0, 0], A_matrix[0, 1], A_matrix[0, 2]), (A_matrix[1, 0], A_matrix[1, 1], A_matrix[1, 2]))}, vector: A_vector{=(A_vector[0], A_vector[1])})}")
        self.failUnlessEqual(str(trafo((matrix([2, 3], "B"), vector(2, "c")), "A")), "A{=(matrix: B{=((B[0, 0], B[0, 1], B[0, 2]), (B[1, 0], B[1, 1], B[1, 2]))}, vector: c{=(c[0], c[1])})}")
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), trafo, (matrix([2, 3]), vector(3)))

    def testAccess(self):
        m = matrix([2, 3])
        v = vector(2)
        t = trafo((m, v))
        self.failUnlessEqual(t.getmatrix(), m)
        self.failUnlessEqual(t.getvector(), v)

    def testMath(self):
        self.failUnlessEqual(str(-trafo([2, 3], "A")), str(trafo((-matrix([2, 3], "A_matrix"), -vector(2, "A_vector")))))
        self.failUnlessEqual(str(trafo([2, 3], "A") + trafo([2, 3], "B")), str(trafo((matrix([2, 3], "A_matrix") + matrix([2, 3], "B_matrix"), vector(2, "A_vector") + vector(2, "B_vector")))))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__add__, 1, trafo([2, 3]))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__add__, trafo([2, 3]), 1)
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__add__, scalar(), trafo([2, 3]))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__add__, trafo([2, 3]), scalar())
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__add__, scalar() + scalar(), trafo([2, 3]))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__add__, trafo([2, 3]), scalar() + scalar())
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__add__, vector(2), trafo([2, 3]))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__add__, trafo([2, 3]), vector(2))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__add__, vector(3), trafo([2, 3]))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__add__, trafo([2, 3]), vector(3))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__add__, matrix([2, 3]), trafo([2, 3]))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__add__, trafo([2, 3]), matrix([2, 3]))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__add__, trafo([2, 3]), trafo([2, 4]))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__add__, trafo([2, 3]), trafo([3, 3]))
        self.failUnlessEqual(str(trafo([2, 3], "A") - trafo([2, 3], "B")), str(trafo((matrix([2, 3], "A_matrix") - matrix([2, 3], "B_matrix"), vector(2, "A_vector") - vector(2, "B_vector")))))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__sub__, 1, trafo([2, 3]))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__sub__, trafo([2, 3]), 1)
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__sub__, scalar(), trafo([2, 3]))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__sub__, trafo([2, 3]), scalar())
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__sub__, scalar() + scalar(), trafo([2, 3]))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__sub__, trafo([2, 3]), scalar() + scalar())
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__sub__, vector(2), trafo([2, 3]))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__sub__, trafo([2, 3]), vector(2))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__sub__, vector(3), trafo([2, 3]))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__sub__, trafo([2, 3]), vector(3))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__sub__, matrix([2, 3]), trafo([2, 3]))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__sub__, trafo([2, 3]), matrix([2, 3]))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__sub__, trafo([2, 3]), trafo([2, 4]))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__sub__, trafo([2, 3]), trafo([3, 3]))
        self.failUnlessEqual(str(trafo([2, 3], "A") * trafo([3, 4], "B")), str(trafo((matrix([2, 3], "A_matrix") * matrix([3, 4], "B_matrix"), vector(2, "A_vector") + matrix([2, 3], "A_matrix") * vector(3, "B_vector")))))
        self.failUnlessEqual(str(2 * trafo([2, 3], "A")), str(trafo((2 * matrix([2, 3], "A_matrix"), 2 * vector(2, "A_vector")))))
        self.failUnlessEqual(str(trafo([2, 3], "A") * 2), str(trafo((matrix([2, 3], "A_matrix") * 2, vector(2, "A_vector") * 2))))
        self.failUnlessEqual(str(scalar() * trafo([2, 3], "A")), str(trafo((scalar() * matrix([2, 3], "A_matrix"), scalar() * vector(2, "A_vector")))))
        self.failUnlessEqual(str(trafo([2, 3], "A") * scalar()), str(trafo((matrix([2, 3], "A_matrix") * scalar(), vector(2, "A_vector") * scalar()))))
        self.failUnlessEqual(str((scalar() + scalar()) * trafo([2, 3], "A")), str(trafo(((scalar() + scalar()) * matrix([2, 3], "A_matrix"), (scalar() + scalar()) * vector(2, "A_vector")))))
        self.failUnlessEqual(str(trafo([2, 3], "A") * (scalar() + scalar())), str(trafo((matrix([2, 3], "A_matrix") * (scalar() + scalar()), vector(2, "A_vector") * (scalar() + scalar())))))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__mul__, vector(2), trafo([2, 3]))
        self.failUnlessEqual(str(trafo([2, 3], "A") * vector(3, "a")), str(matrix([2, 3], "A_matrix") * vector(3, "a") + vector(2, "A_vector")))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__mul__, trafo([2, 3]), vector(2))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__mul__, matrix([3, 2]), trafo([2, 3]))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__mul__, trafo([2, 3]), matrix([3, 2]))
        self.failUnlessEqual(str(trafo([2, 3], "A") / 2.0), str(trafo((matrix([2, 3], "A_matrix") / 2.0, vector(2, "A_vector") / 2.0))))
        self.failUnlessEqual(str(trafo([2, 3], "A") / 2), str(trafo((matrix([2, 3], "A_matrix") / 2, vector(2, "A_vector") / 2))))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__div__, scalar(), trafo([2, 3]))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__div__, scalar() + scalar(), trafo([2, 3]))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__div__, vector(1), trafo([2, 3]))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__div__, matrix([2, 3]), trafo([2, 3]))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__div__, trafo([2, 3]), scalar())
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__div__, trafo([2, 3]), scalar() + scalar())
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__div__, trafo([2, 3]), vector(1))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__div__, trafo([2, 3]), matrix([2, 3]))
        self.failUnlessRaises((RuntimeError, AttributeError, TypeError), operator.__div__, trafo([2, 3]), trafo([2, 3]))

class SolverTestCase(unittest.TestCase):

    def testScalars(self):
        solver = Solver()
        a = scalar()
        solver.eq(2 + a, 3)
        self.failUnlessAlmostEqual(float(a), 1)
        self.failUnlessRaises(RuntimeError, solver.eq, 2 + a, 3) # the solver is broken after that
        solver = Solver()
        a = scalar()
        b = scalar()
        solver.eq(2*a, b)
        solver.eq(2 + a + b, 5)
        self.failUnlessAlmostEqual(float(a), 1)
        self.failUnlessAlmostEqual(float(b), 2)
        a = scalar()
        b = scalar()
        solver.eq(a, b)
        self.failUnlessRaises(Exception, solver.eq, a, b)

    def testVector(self):
        solver = Solver()
        a = vector(2)
        solver.eq(vector([1, 2]) + a, vector([3, 3]))
        self.failUnlessAlmostEqual(float(a.x), 2)
        self.failUnlessAlmostEqual(float(a.y), 1)
        b = vector(2)
        solver.eq(a*b) # => solver.eq(a*b, 0)
        solver.eq(b.y, 2)
        self.failUnlessAlmostEqual(float(b.x), -1)

    def testMatrix(self):
        solver = Solver()
        A = matrix([2, 2])
        solver.eq(A*vector([1, -1]), vector([2, 0]))
        solver.eq(A*vector([1, 1]), vector([0, 2]))
        self.failUnlessAlmostEqual(float(A[0, 0]), 1)
        self.failUnlessAlmostEqual(float(A[0, 1]), -1)
        self.failUnlessAlmostEqual(float(A[1, 0]), 1)
        self.failUnlessAlmostEqual(float(A[1, 1]), 1)
        B = matrix([2, 2])
        solver.eq(A*B, identitymatrix(2))
        self.failUnlessAlmostEqual(float(B[0, 0]), 0.5)
        self.failUnlessAlmostEqual(float(B[0, 1]), 0.5)
        self.failUnlessAlmostEqual(float(B[1, 0]), -0.5)
        self.failUnlessAlmostEqual(float(B[1, 1]), 0.5)
        self.failUnlessAlmostEqual(float((B * vector([2, 0]))[0]), 1)
        self.failUnlessAlmostEqual(float((B * vector([2, 0]))[1]), -1)
        self.failUnlessAlmostEqual(float((B * vector([0, 2]))[0]), 1)
        self.failUnlessAlmostEqual(float((B * vector([0, 2]))[1]), 1)

    def testTrafo(self):
        solver = Solver()
        A = trafo([2, 2])
        solver.eq(A.getmatrix()[0, 0], A.getmatrix()[1, 1]) # restrict to rotation + scaling + translation
        solver.eq(A.getmatrix()[0, 1], -A.getmatrix()[1, 0])
        solver.eq(A*vector([0, 1]), vector([-1, 2]))
        solver.eq(A*vector([1, 0]), vector([1, 4]))
        self.failUnlessEqual(float((A * vector([0, 0]))[0]), 1)
        self.failUnlessEqual(float((A * vector([0, 0]))[1]), 2)
        self.failUnlessEqual(float(A.getmatrix()[0, 0]), 0)
        self.failUnlessEqual(float(A.getmatrix()[0, 1]), -2)
        self.failUnlessEqual(float(A.getmatrix()[1, 0]), 2)
        self.failUnlessEqual(float(A.getmatrix()[1, 1]), 0)
        B = trafo([2, 2])
        solver.eq(A*B, identitytrafo(2))
        self.failUnlessEqual(float((B * vector([0, 0]))[0]), -1)
        self.failUnlessEqual(float((B * vector([0, 0]))[1]), 0.5)
        self.failUnlessEqual(float(B.getmatrix()[0, 0]), 0)
        self.failUnlessEqual(float(B.getmatrix()[0, 1]), 0.5)
        self.failUnlessEqual(float(B.getmatrix()[1, 0]), -0.5)
        self.failUnlessEqual(float(B.getmatrix()[1, 1]), 0)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# -*- coding: ISO-8859-1 -*-
import sys; sys.path[:0] = ["../.."]

import pyx
from pyx import *
from pyx.font.font import PDFTimesBold
from pyx.pdfextra import *


def texttest():
    c = canvas.canvas()
    d = 0.0

    llx, lly, w, h = 0, 0, 5, 1
    c.text(llx, lly+h, "first textfield:", [text.vshift(-0.5)])
    c.stroke(path.rect(llx-d, lly-d, w+2*d, h+2*d))
    c.insert(textfield(llx, lly, w, h, name="textfield", defaultvalue="hallo", multiline=1,
      font=PDFTimesBold))

    llx, lly, w, h = 0, -2, 1, 1
    c.text(llx, lly+h, "second textfield:", [text.vshift(-0.5)])
    c.stroke(path.rect(llx-d, lly-d, w+2*d, h+2*d))
    c.insert(textfield(llx, lly, w, h, name="another"))

    return c

def checkboxtest():
    c = canvas.canvas()

    c.text(0, 0, "first checkbox:", [text.vshift(-0.5)])
    c.insert(checkbox(0, -0.5, name="checkbox", defaulton=1))

    return c

def radiotest():
    c = canvas.canvas()

    c.text(0, 0, "first radiobuttons:", [text.vshift(-0.5)])
    pos = [(0, -0.5), (0, -1.0), (0, -1.5)]
    values = ["One", "Two", "Three"]
    for (x, y), value in zip(pos, values):
        c.text(x+12*unit.x_pt, y, value)
    c.insert(radiobuttons(pos, name="button", values=values, defaultvalue=values[0]))

    # XXX : default entry is activated with the others

    return c

def choicetest():
    c = canvas.canvas()
    d = 0.0

    llx, lly, w, h = 0, 0, 5, 1
    values = ["One", "Two", "Three"]
    c.text(llx, lly+h, "first choicefield:", [text.vshift(-0.5)])
    c.stroke(path.rect(llx-d, lly-d, w+2*d, h+2*d))
    c.insert(choicefield(llx, lly, w, h, name="choicefield", values=values, defaultvalue=values[0],
        font=PDFTimesBold))

    return c

# write all test into one canvas
c = canvas.canvas()
for cc in [texttest(), checkboxtest(), radiotest(), choicetest()]:
    if not c:
        t = []
    else:
        t = [trafo.translate(0, c.bbox().bottom() - cc.bbox().top() - 1)]
    c.insert(cc, t)
    #c.stroke(cc.bbox().path(), t + [color.rgb.red, style.linestyle.dotted])

# test the transformation behaviour:
cc = canvas.canvas([trafo.scale(0.3)])#, trafo.rotate(90)])
cc.fill(c.bbox().path(), [color.gray(0.9)])
cc.insert(c)
d = document.document([document.page(cc, bboxenlarge=1)])
d.writePDFfile("test_pdfextra", compress=False)

# vim:syntax=python
# vim:fdm=marker:fmr=<<<,>>>:



#!/usr/bin/env python
import sys; sys.path[:0] = ["../.."]

from pyx import *

trafo1 = trafo.rotate(45)
trafo2 = trafo.translate(1, 1)
clip = canvas.clip(path.rect(-1, -1, 2, 2))
content = canvas.canvas()
content.fill(path.rect(-2, -2, 2, 2), [color.rgb.red])
content.fill(path.rect(-2, 0, 2, 2), [color.rgb.green])
content.fill(path.rect(0, -2, 2, 2), [color.rgb.blue])
content.fill(path.rect(0, 0, 2, 2), [color.rgb.black])

def apply1(*args):
    c = canvas.canvas(args)
    c.insert(content)
    return c

def apply2(*args):
    c = sub = canvas.canvas()
    for arg in reversed(args):
        sub = sub.insert(canvas.canvas([arg]))
    sub.insert(content)
    return c

c = canvas.canvas()

c.insert(apply1(clip, trafo1, trafo2), [trafo.translate(0, 0)])
c.stroke(apply1(clip, trafo1, trafo2).bbox().rect(), [trafo.translate(0, 0)])

c.insert(apply2(clip, trafo1, trafo2), [trafo.translate(0, 5)])
c.stroke(apply2(clip, trafo1, trafo2).bbox().rect(), [trafo.translate(0, 5)])

c.insert(apply1(clip, trafo2, trafo1), [trafo.translate(5, 0)])
c.stroke(apply1(clip, trafo2, trafo1).bbox().rect(), [trafo.translate(5, 0)])

c.insert(apply2(clip, trafo2, trafo1), [trafo.translate(5, 5)])
c.stroke(apply2(clip, trafo2, trafo1).bbox().rect(), [trafo.translate(5, 5)])

c.insert(apply1(trafo1, clip, trafo2), [trafo.translate(10, 0)])
c.stroke(apply1(trafo1, clip, trafo2).bbox().rect(), [trafo.translate(10, 0)])

c.insert(apply2(trafo1, clip, trafo2), [trafo.translate(10, 5)])
c.stroke(apply2(trafo1, clip, trafo2).bbox().rect(), [trafo.translate(10, 5)])

c.insert(apply1(trafo2, clip, trafo1), [trafo.translate(15, 0)])
c.stroke(apply1(trafo2, clip, trafo1).bbox().rect(), [trafo.translate(15, 0)])

c.insert(apply2(trafo2, clip, trafo1), [trafo.translate(15, 5)])
c.stroke(apply2(trafo2, clip, trafo1).bbox().rect(), [trafo.translate(15, 5)])

c.insert(apply1(trafo1, trafo2, clip), [trafo.translate(20, 0)])
c.stroke(apply1(trafo1, trafo2, clip).bbox().rect(), [trafo.translate(20, 0)])

c.insert(apply2(trafo1, trafo2, clip), [trafo.translate(20, 5)])
c.stroke(apply2(trafo1, trafo2, clip).bbox().rect(), [trafo.translate(20, 5)])

c.insert(apply1(trafo2, trafo1, clip), [trafo.translate(25, 0)])
c.stroke(apply1(trafo2, trafo1, clip).bbox().rect(), [trafo.translate(25, 0)])

c.insert(apply2(trafo2, trafo1, clip), [trafo.translate(25, 5)])
c.stroke(apply2(trafo2, trafo1, clip).bbox().rect(), [trafo.translate(25, 5)])

c.writeEPSfile()
c.writePDFfile()
c.writeSVGfile()



#!/bin/python3
from pyx import *
c = canvas.canvas()
def bluedot(x, y):
    c.fill(path.circle_pt(x, y, 2), [color.rgb.blue])
c.stroke(path.path(path.moveto(0, 0), path.arct(2, 0, 1.5, 1, 0.5)))
def bluedot(x, y):
    c.fill(path.circle(x, y, 0.03), [color.rgb.blue])
bluedot(0, 0)
bluedot(2, 0)
bluedot(1.5, 1)
margin=0.1
c.text(0-margin, 0, r"current point $(0, 0)$", [text.halign.right, text.valign.baseline])
c.text(2+margin, 0, r"$(2, 0)$", [text.halign.left, text.valign.baseline])
c.text(1.5-margin, 1, r"$(1.5, 1)$", [text.halign.right, text.valign.baseline])
c.writePDFfile()



#!/usr/bin/env python
import sys; sys.path[:0] = ["../.."]
import warnings, math
from pyx import *
from pyx.deformer import *
from pyx import normpath

#####  helpers  ##############################################################

def bboxrect(cmd):
   return cmd.bbox().enlarged(5*unit.t_mm).rect()

def dotest(c, x, y, test):
   c2 = c.insert(canvas.canvas([trafo.translate(x, y)]))
   eval("%s(c2)" % test)
   c.stroke(bboxrect(c2))

def drawpathwbbox(c, p):
    c.stroke(p, [color.rgb.red])
    np = p.normpath()
    c.stroke(np, [color.rgb.green, style.linestyle.dashed])
    c.stroke(bboxrect(p))

#####  tests  ################################################################

def testcycloid(c):

    # dependence on turnangle
    p = path.line(0, 0, 3, 0)
    c.stroke(p, [style.linewidth.THIN])
    cyc = cycloid(halfloops=3, skipfirst=0.5, skiplast=0.5, curvesperhloop=2)
    c.stroke(p, [cyc(turnangle=00)])
    c.stroke(p, [cyc(turnangle=22), color.rgb.red])
    c.stroke(p, [cyc(turnangle=45), color.rgb.green])
    c.stroke(p, [cyc(turnangle=67), color.rgb.blue])
    c.stroke(p, [cyc(turnangle=90), color.cmyk.Cyan])

    # dependence on curvesperloop
    p = path.curve(5, 0, 8, 0, 6, 4, 9, 4)
    c.stroke(p)
    cyc = cycloid(halfloops=16, skipfirst=0, skiplast=0, curvesperhloop=1)
    c.stroke(p, [cyc(curvesperhloop=2)])
    c.stroke(p, [cyc(curvesperhloop=3), color.rgb.red])
    c.stroke(p, [cyc(curvesperhloop=4), color.rgb.green])
    c.stroke(p, [cyc(curvesperhloop=10), color.rgb.blue])

    # extremely curved path
    p = path.curve(0,2, 0.5,5, 1,6, 2,2)
    c.stroke(p)
    cyc = cycloid(radius=0.7, halfloops=7, skipfirst=0, skiplast=0, curvesperhloop=1)
    c.stroke(p, [cyc(curvesperhloop=2)])
    c.stroke(p, [cyc(curvesperhloop=3), color.rgb.red])
    c.stroke(p, [cyc(curvesperhloop=4), color.rgb.green])
    c.stroke(p, [cyc(curvesperhloop=50), color.rgb.blue])


def testcornersmoothed(c):
    p = path.path(
      path.moveto(0,0),
      path.lineto(3,0),
      path.lineto(5,7),
      path.curveto(0,10, -2,8, 0,6),
      path.lineto(0,4),
      # horrible overshooting with obeycurv=1
      #path.lineto(-4,4), path.curveto(-7,5, -4,2, -5,2),
      path.lineto(-4,3), path.curveto(-7,5, -4,2, -5,2),
      #path.arct(-6,4, -5,1, 1.5),
      #path.arc(-5, 3, 0.5, 0, 180),
      path.lineto(-5,1),
      path.lineto(-0.2,0.2),
      path.closepath()
    ) + path.circle(0,1,2)

    c.stroke(p, [color.gray(0.8), style.linewidth.THICk])
    c.stroke(p.normpath(), [color.gray(0.8), style.linewidth.THICk])
    c.stroke(p, [cornersmoothed(radius=0.85, softness=1, obeycurv=1), style.linewidth.Thin])
    c.stroke(p, [cornersmoothed(radius=0.85, softness=1, obeycurv=0), color.rgb.red])
    c.stroke(p, [cornersmoothed(radius=0.20, softness=1, obeycurv=0), color.rgb.green])
    c.stroke(p, [cornersmoothed(radius=1.20, softness=1, obeycurv=0), color.rgb.blue])

    p = path.path(
      path.moveto(0,10),
      path.curveto(1,10, 4,12, 2,11),
      path.curveto(4,8, 4,12, 0,11)
    )
    c.stroke(p, [color.gray(0.8), style.linewidth.THICk])
    c.stroke(p.normpath(), [color.gray(0.8), style.linewidth.THICk])
    c.stroke(p, [cornersmoothed(radius=0.85, softness=1, obeycurv=1), style.linewidth.Thick])
    c.stroke(p, [cornersmoothed(radius=0.85, softness=1, obeycurv=0), color.rgb.red])
    c.stroke(p, [cornersmoothed(radius=0.20, softness=1, obeycurv=0), color.rgb.green])
    c.stroke(p, [cornersmoothed(radius=1.20, softness=1, obeycurv=0), color.rgb.blue])


def hard_test(c, pth, dist, pardef, move=(0, 0), label=None, dotests=[True]*4, checklens=[None]*4):
    print("hard test of parallel: ", label)
    tr = trafo.translate(*move)
    if label is not None:
        c.text(0.5*(pth.bbox().left() + pth.bbox().right()),
               0.5*(pth.bbox().bottom() + pth.bbox().top()), label, [tr])
    c.stroke(pth, [tr])
    pth_rev = pth.reversed()
    dists = [dist, -dist, -dist, dist]
    paths = [pth, pth_rev, pth, pth_rev]
    styles = [[color.rgb.red], [style.linestyle.dashed], [color.rgb.blue], [color.rgb.green, style.linestyle.dashed]]
    for i, d, p, dotest, assertlens, st in zip(range(4), dists, paths, dotests, checklens, styles):
        if dotest:
            pl = pardef(distance=d).deform(p)
            c.stroke(pl, st + [tr])#, deco.shownormpath()])
            if assertlens is None:
                print([len(nsp) for nsp in pl])
            else:
                if len(pl) != len(assertlens):
                    print("WARNING: wrong number of nsp (%d instead of expected %d) in test %d" % (len(pl), len(assertlens), i))
                for j, (n_nspitems, nsp) in enumerate(zip(assertlens, pl)):
                    if len(nsp) != n_nspitems:
                        print("WARNING: wrong number of nsp items (%d instead of expected %d) in test %d, nsp number %d" % (len(nsp), n_nspitems, i, j))

def testparallel_1(c):

    # HARD TESTS of elementary geometry:
    #
    # test for correct skipping of short ugly pieces:
    move = (-2, 0)
    p = path.path(path.moveto(0, 1), path.lineto(10, 0.3), path.lineto(12, 0), path.lineto(0, 0))
    p.append(path.closepath())
    hard_test(c, p, 0.1, parallel(0.0), move, "A", checklens=[[15],[15],[4],[4]])
    hard_test(c, p, 0.25, parallel(0.0, sharpoutercorners=True), move, "A", checklens=[[12],[12],[3],[3]])
    hard_test(c, p, 0.55, parallel(0.0), move, "A", checklens=[[15],[15],[],[]])

    # test non-intersecting/too short neighbouring pathels
    move = (0, 4)
    p = path.curve(0,0, 0,1, 1,2, 2,0)
    p.append(path.lineto(2.1, 0.1))
    p.append(path.lineto(1.6, -2))
    p.append(path.lineto(2.1, -2))
    p.append(path.lineto(-0.15, 0))
    p.append(path.closepath())
    hard_test(c, p, 0.02, parallel(0.0, sharpoutercorners=1), move, "B", checklens=[[3],[3],[11],[11]])
    hard_test(c, p, 0.06, parallel(0.0), move, "B", checklens=[[3],[3],[10],[9]]) # difference is due to rounding in path._arctobeziers
    hard_test(c, p, 0.3, parallel(0.0), move, "B", checklens=[[25],[25],[5],[5]])
    hard_test(c, p, 0.3, parallel(0.0, sharpoutercorners=1), move, "B", checklens=[[22],[22],[5],[5]])

    # test extreme precision:
    move = (3.5, 2)
    p = path.curve(0,0, 0,1, 1,1, 1,0)
    p.append(path.closepath())
    hard_test(c, p, 0.1, parallel(0.0), move, "C", checklens=[[8],[8],[2],[2]])
    hard_test(c, p, 0.1, parallel(0.0, relerr=1e-15, checkdistanceparams=[0.5]), move, "C", checklens=[[23],[23],[15],[15]])

    # test for numeric instabilities:
    move = (6, 2)
    p = path.curve(0,0, 1,1, 1,1, 2,0)
    p.append(path.closepath())
    hard_test(c, p, 0.1, parallel(0.0, relerr=0.15, checkdistanceparams=[0.5]), move, "D", checklens=[[11],[11],[3],[3]])
    hard_test(c, p, 0.3, parallel(0.0), move, "D", checklens=[[11],[11],[3],[3]])

    # test for an empty parallel path:
    move = (5, 5)
    p = path.circle(0, 0, 0.5).normpath()
    nspitems = [nspitem for nspitem in p[0] if isinstance(nspitem, normpath.normcurve_pt)]
    nspitems[-1] = nspitems[-1].modifiedend_pt(*nspitems[0].atbegin_pt())
    p = normpath.normpath([normpath.normsubpath(nspitems, closed=True)])
    hard_test(c, p, 0.55, parallel(0.0), move, "E", checklens=[[],[],[9],[9]])
    hard_test(c, p, 0.4999, parallel(0.0, relerr=1.0e-15), move, "E", checklens=[[1,14,8],[1,14],[17],[17]])

    # a degenerate path:
    move = (13, 3)
    p = path.curve(0,0, 0,-5, 0,1, 0,0.5)
    hard_test(c, p, 0.10, parallel(0.0, dointersection=False), move, "F", dotests=[True,True,False,False], checklens=[[13],[12],[13],[12]])
    hard_test(c, p, 0.13, parallel(0.0, dointersection=False), move, "F", dotests=[False,False,True,True], checklens=[[13],[12],[13],[12]])

    # test for too big curvatures in the middle:
    move = (9, 2.5)
    p = path.curve(0,0, 1,1, 1,1, 2,0)
    hard_test(c, p, 0.4, parallel(0.0, relerr=1.0e-2), move, "G", checklens=[[2],[2],[4],[4]])
    hard_test(c, p, 0.6, parallel(0.0, relerr=1.0e-2), move, "G", checklens=[[2],[2],[4],[4]])
    hard_test(c, p, 0.8, parallel(0.0, relerr=1.0e-2), move, "G", checklens=[[2],[2],[4],[4]])
    hard_test(c, p, 1.7, parallel(0.0, relerr=1.0e-2), move, "G", checklens=[[2],[2],[1,1],[1,1]])

    # deformation of the deformation:
    move = (11, 6)
    p = path.curve(-1,0, 0,1, 0,1, 1,0).normpath()
    c.stroke(p, [trafo.translate(*move), color.gray(0.8)])
    dist = unit.u_pt/p.curvature_pt([normpath.normpathparam(p,0,0.5)])[0]
    p = parallel(dist, relerr=1.0e-2, dointersection=0).deform(p)
    assert len(p) == 2
    assert len(p[0]) == len(p[1]) == 2
    c.stroke(p, [trafo.translate(*move), color.gray(0.8)])
    hard_test(c, p, -dist, parallel(0.0), move, "H", checklens=[[4],[4],[2],[2]])

    # test for infinite curvature in the middle:
    move = (11, 8.5)
    p = path.curve(0,0, 1,1, 0,1, 1,0).normpath()
    hard_test(c, p, 0.2, parallel(0.0), move, "I", checklens=[[1,1],[1,1],[2],[2]])
    hard_test(c, p, 1.5, parallel(0.0), move, "I", checklens=[[1,1],[1,1],[7],[7]])
    #ard_test(c, p, 5.0, parallel(0.0), move, "I", checklens=[[],[],[7],[7]])

    # test for infinite curvature at the end:
    move = (-2, 13)
    p = path.curve(0,0, 0.5,0.5, 0.75,0.5, 0.75,0.5)
    hard_test(c, p, 0.1, parallel(0.0, relerr=1.0e-4), move, "J", checklens=[[5],[5],[5],[5]])
    # test for infinite curvature when the path goes on
    # XXX this is not correctly detected if rlineto(1,0). Expect two nsp
    p.append(path.rlineto(0, 1))
    hard_test(c, p, 0.22, parallel(0.0, relerr=1.0e-4), move, "J", checklens=[[4],[4],[3,1],[1,3]])

    # test for too big curvature in the middle, the non-intersecting case
    move = (0, 8)
    p = path.curve(-1,-1, 4,4, -4,4, 1,-1).normpath()
    dist = unit.u_pt * (1.0/p.curvature_pt([normpath.normpathparam(p, 0, 0.5)])[0])
    hard_test(c, p, 1.5*dist, parallel(0.0, dointersection=False), move, "K", checklens=[[1,1],[1,1],[2],[2]])
    hard_test(c, p, dist+normpath._epsilon, parallel(0.0, dointersection=False, relerr=1.0e-5), move, "K", checklens=[[11,11],[11,11],[16],[16]])
    hard_test(c, p, dist+0*normpath._epsilon, parallel(0.0, dointersection=False, relerr=1.0e-5), move, "K", checklens=[[22],[22],[16],[16]])

    # what is to be done with the arcs running around corners?
    # for dist=1.1 they do not even intersect with the original path:
    # TODO
    move = (8, 14)
    p = path.rect(0, 0, 1, 1).normpath()
    hard_test(c, p, 1.1, parallel(0.0), move, "L", dotests=[True,True,False,False], checklens=[[20],[20],None,None])
    hard_test(c, p, 0.55, parallel(0.0), move, "L", dotests=[True,True,False,False], checklens=[[28],[28],None,None])

    # very small lines approximating a curve
    # make sure that there is no "dust" remaining from the many intersections
    move = (3, 9)
    N = 12
    angle = 0.5*math.pi / (N-1.0)
    p = path.path(path.moveto(0, 0))
    for n in range(N):
        p.append(path.lineto(2+math.sin(n*angle), 1-math.cos(n*angle)))
    p.append(path.rlineto(0, 1))
    p.append(path.rlineto(2, 0))
    p.append(path.rlineto(0, -2))
    p = p.transformed(trafo.translate(0, 0.05))
    p = p << p.transformed(trafo.scale(1, -1)).reversed()
    hard_test(c, p, 0.6, parallel(0.0), move, "M", checklens=[[43],[43],[29],[29]])

    # intersections where paths are parallel (in the middle of a nsp-item)
    move = (4, 15)
    c.text(move[0], move[1]+0.2, "N")
    tr = trafo.translate(*move)
    par = parallel(0.1)
    par.dist_pt = unit.topt(par.distance)
    c1 = path.curve(-2,-1, -2,1, 2,-1, 2,1).transformed(tr).normpath()
    c2 = path.curve(-2,-1, -2,1/3.0, 2,1/3.0, 2,-1).transformed(tr).normpath()
    l = path.line(-2,0, 2,0).transformed(tr).normpath()
    for p in [c1, c2, l]:
        c.stroke(p)
    p = (c1 + l).normpath()
    try:
        par._can_continue(mynormpathparam(p,0,0,0.5), mynormpathparam(p,1,0,0.5))
    except IntersectionError as e:
        assert str(e) == "Cannot determine whether curves intersect (parallel and equally curved)"
    p = c2 + l
    assert par._can_continue(mynormpathparam(p,0,0,0.5), mynormpathparam(p,1,0,0.5))
    p = c2 + l.reversed()
    assert par._can_continue(mynormpathparam(p,0,0,0.5), mynormpathparam(p,1,0,0.5))
    p = c2.reversed() + l
    assert not par._can_continue(mynormpathparam(p,0,0,0.5), mynormpathparam(p,1,0,0.5))
    p = c2.reversed() + l.reversed()
    assert not par._can_continue(mynormpathparam(p,0,0,0.5), mynormpathparam(p,1,0,0.5))
    p = c1 + c2
    assert not par._can_continue(mynormpathparam(p,0,0,0.5), mynormpathparam(p,1,0,0.5))
    p = c2 + c1
    assert par._can_continue(mynormpathparam(p,0,0,0.5), mynormpathparam(p,1,0,0.5))
    # intersections where paths are parallel (between nsp-items)
    c11, c12 = c1.split([normpath.normpathparam(c1, 0, 0.5)])
    c21, c22 = c2.split([normpath.normpathparam(c2, 0, 0.5)])
    l1, l2 = l.split([normpath.normpathparam(l, 0, 0.5)])
    p = (c21 << l2 + l1 << c22).normpath() # curve-line + line-curve
    assert par._can_continue(mynormpathparam(p,1,0,1), mynormpathparam(p,0,1,0)) # line -> line
    assert not par._can_continue(mynormpathparam(p,0,0,1), mynormpathparam(p,1,1,0)) # curve -> curve

    # intersections where paths are parallel (in the middle of a nsp-item)
    move = (12, 13)
    p1 = path.path(path.moveto(0,0), path.curveto(0,3, 1,5, 1,0)).normpath()
    params1, params2 = p1.intersect(path.line(0.5,-1, 0.5,5))
    p11, p12 = p1.split(params1)
    p11.append(path.rlineto(0,-2))
    p = p11 << p12
    p.append(path.closepath())
    hard_test(c, p, 0.55, parallel(0.0), move, "O", dotests=[True,True,False,False], checklens=[[10],[10],None,None])

    # jump to too large curvature between nspitems
    move = (-2, 17)
    p = path.path(path.moveto(0,0), path.arc(2,0.5,0.5,-90,90), path.lineto(0,1)).normpath()
    hard_test(c, p, 0.55, parallel(0.0), move, "P", dotests=[True,True,True,True], checklens=[[1,1],[1,1],[7],[7]])

def testparallel_2(c):

    # a case with several dangers:
    #  - double intersection pairs
    #  - intersections with angle 0
    #  - inner intersections that have to be cut away by _between_paths
    move = (10, 2)
    nsp = normpath.normsubpath([# <<<
    #ormpath.normline_pt(-87.5005, 184.037, -86.2747, 184.198),
    #ormpath.normline_pt(-86.2747, 184.198, -85.0394, 184.252),
    #ormpath.normline_pt(-85.0394, 184.252, -83.8041, 184.198),
    #ormpath.normline_pt(-83.8041, 184.198, -82.5782, 184.037),
    #ormpath.normline_pt(-82.5782, 184.037, -81.3711, 183.769),
    #ormpath.normline_pt(-81.3711, 183.769, -80.1918, 183.397),
    #ormpath.normline_pt(-80.1918, 183.397, -79.0495, 182.924),
    #ormpath.normline_pt(-79.0495, 182.924, -77.9528, 182.353),
    #ormpath.normline_pt(-77.9528, 182.353, -76.9099, 181.689),
    #ormpath.normline_pt(-76.9099, 181.689, -75.929, 180.936),
    #ormpath.normline_pt(-75.929, 180.936, -75.0174, 180.101),
    #ormpath.normline_pt(-75.0174, 180.101, -74.1821, 179.189),
    #ormpath.normline_pt(-74.1821, 179.189, -73.4293, 178.208),
    #ormpath.normline_pt(-73.4293, 178.208, -72.765, 177.165),
    #ormpath.normline_pt(-72.765, 177.165, -72.1941, 176.069),
    #ormpath.normline_pt(-72.1941, 176.069, -71.7209, 174.926),
    #ormpath.normline_pt(-71.7209, 174.926, -71.3491, 173.747),
    #ormpath.normline_pt(-71.3491, 173.747, -71.0815, 172.54),
    #ormpath.normline_pt(-71.0815, 172.54, -70.9201, 171.314),
    #ormpath.normline_pt(-70.9201, 171.314, -70.8661, 170.079),
    #ormpath.normline_pt(-70.8661, 170.079, -70.9201, 168.843),
    #ormpath.normline_pt(-70.9201, 168.843, -71.0815, 167.618),
    #ormpath.normline_pt(-71.0815, 167.618, -71.3491, 166.41),
    #ormpath.normline_pt(-71.3491, 166.41, -71.7209, 165.231),
    #ormpath.normline_pt(-71.7209, 165.231, -72.1941, 164.089),
    #ormpath.normline_pt(-72.1941, 164.089, -72.765, 162.992),
    #ormpath.normline_pt(-72.765, 162.992, -73.4293, 161.949),
    #ormpath.normline_pt(-73.4293, 161.949, -74.1821, 160.968),
    #ormpath.normline_pt(-74.1821, 160.968, -75.0174, 160.057),
    #ormpath.normline_pt(-75.0174, 160.057, -75.929, 159.221),
    #ormpath.normline_pt(-75.929, 159.221, -76.9099, 158.469),
    #ormpath.normline_pt(-76.9099, 158.469, -77.9528, 157.804),
    #ormpath.normline_pt(-77.9528, 157.804, -77.9528, -28.3465),
    #ormpath.normline_pt(-77.9528, -28.3465, -77.9797, -28.9641),
    #ormpath.normline_pt(-77.9797, -28.9641, -78.0604, -29.577),
    #ormpath.normline_pt(-78.0604, -29.577, -78.1942, -30.1806),
    #ormpath.normline_pt(-78.1942, -30.1806, -78.3801, -30.7702),
    #ormpath.normline_pt(-78.3801, -30.7702, -78.6167, -31.3414),
    #ormpath.normline_pt(-78.6167, -31.3414, -78.9022, -31.8897),
    #ormpath.normline_pt(-78.9022, -31.8897, -79.2344, -32.4111),
    #ormpath.normline_pt(-79.2344, -32.4111, -79.6107, -32.9016),
    #ormpath.normline_pt(-79.6107, -32.9016, -80.0284, -33.3574),
    #ormpath.normline_pt(-80.0284, -33.3574, -80.4842, -33.7751),
    #ormpath.normline_pt(-80.4842, -33.7751, -80.9747, -34.1514),
    #ormpath.normline_pt(-80.9747, -34.1514, -81.4961, -34.4836),
    #ormpath.normline_pt(-81.4961, -34.4836, -82.0444, -34.7691),
    #ormpath.normline_pt(-82.0444, -34.7691, -82.6156, -35.0057),
    #ormpath.normline_pt(-82.6156, -35.0057, -83.2052, -35.1916),
    #ormpath.normline_pt(-83.2052, -35.1916, -83.8088, -35.3254),
    #ormpath.normline_pt(-83.8088, -35.3254, -84.3307, -35.3941),
    #ormpath.normline_pt(-84.3307, -35.3941, -84.3307, -169.37),
    #ormpath.normline_pt(-84.3307, -169.37, -28.423, -169.37),
    #ormpath.normline_pt(-28.423, -169.37, 0, -156.252),
    #ormpath.normline_pt(0, -156.252, 0, -157.813),
    #ormpath.normline_pt(0, -157.813, -28.1117, -170.787),
    #ormpath.normline_pt(-28.1117, -170.787, -150, -170.787),
    normpath.normline_pt(-150, -170.787, -361.417, -170.787), # start
    normpath.normline_pt(-361.417, -170.787, -361.417, -169.37),
    normpath.normline_pt(-361.417, -169.37, -299.055, -169.37),
    normpath.normline_pt(-299.055, -169.37, -299.055, -141.732),
    normpath.normline_pt(-299.055, -141.732, -297.638, -141.732),
    normpath.normline_pt(-297.638, -141.732, -297.638, -143.15),
    normpath.normline_pt(-297.638, -143.15, -297.638, -169.37),
    normpath.normline_pt(-297.638, -169.37, -270.709, -169.37),
    normpath.normline_pt(-270.709, -169.37, -270.709, -143.15),
    normpath.normline_pt(-270.709, -143.15, -297.638, -143.15),
    normpath.normline_pt(-297.638, -143.15, -297.638, -141.732),
    normpath.normline_pt(-297.638, -141.732, -284.882, -141.732),
    normpath.normline_pt(-284.882, -141.732, -284.882, -113.386),
    normpath.normline_pt(-284.882, -113.386, -283.465, -113.386),
    normpath.normline_pt(-283.465, -113.386, -283.465, -114.803),
    normpath.normline_pt(-283.465, -114.803, -283.465, -141.732),
    normpath.normline_pt(-283.465, -141.732, -269.291, -141.732),
    normpath.normline_pt(-269.291, -141.732, -269.291, -169.37),
    normpath.normline_pt(-269.291, -169.37, -242.362, -169.37),
    normpath.normline_pt(-242.362, -169.37, -242.362, -141.732),
    normpath.normline_pt(-242.362, -141.732, -240.945, -141.732),
    normpath.normline_pt(-240.945, -141.732, -240.945, -143.15),
    normpath.normline_pt(-240.945, -143.15, -240.945, -169.37),
    normpath.normline_pt(-240.945, -169.37, -214.016, -169.37),
    normpath.normline_pt(-214.016, -169.37, -214.016, -143.15),
    normpath.normline_pt(-214.016, -143.15, -240.945, -143.15),
    normpath.normline_pt(-240.945, -143.15, -240.945, -141.732),
    normpath.normline_pt(-240.945, -141.732, -228.189, -141.732),
    normpath.normline_pt(-228.189, -141.732, -228.189, -114.803),
    normpath.normline_pt(-228.189, -114.803, -283.465, -114.803),
    normpath.normline_pt(-283.465, -114.803, -283.465, -113.386),
    normpath.normline_pt(-283.465, -113.386, -226.772, -113.386),
    normpath.normline_pt(-226.772, -113.386, -226.772, -141.732),
    normpath.normline_pt(-226.772, -141.732, -212.598, -141.732),
    normpath.normline_pt(-212.598, -141.732, -212.598, -169.37),
    normpath.normline_pt(-212.598, -169.37, -150, -169.37), # end
    #ormpath.normline_pt(-150, -169.37, -85.748, -169.37),
    #ormpath.normline_pt(-85.748, -169.37, -85.748, -35.3941),
    #ormpath.normline_pt(-85.748, -35.3941, -86.2699, -35.3254),
    #ormpath.normline_pt(-86.2699, -35.3254, -86.8735, -35.1916),
    #ormpath.normline_pt(-86.8735, -35.1916, -87.4631, -35.0057),
    #ormpath.normline_pt(-87.4631, -35.0057, -88.0343, -34.7691),
    #ormpath.normline_pt(-88.0343, -34.7691, -88.5827, -34.4836),
    #ormpath.normline_pt(-88.5827, -34.4836, -89.1041, -34.1514),
    #ormpath.normline_pt(-89.1041, -34.1514, -89.5946, -33.7751),
    #ormpath.normline_pt(-89.5946, -33.7751, -90.0504, -33.3574),
    #ormpath.normline_pt(-90.0504, -33.3574, -90.468, -32.9016),
    #ormpath.normline_pt(-90.468, -32.9016, -90.8444, -32.4111),
    #ormpath.normline_pt(-90.8444, -32.4111, -91.1765, -31.8897),
    #ormpath.normline_pt(-91.1765, -31.8897, -91.462, -31.3414),
    #ormpath.normline_pt(-91.462, -31.3414, -91.6986, -30.7702),
    #ormpath.normline_pt(-91.6986, -30.7702, -91.8845, -30.1806),
    #ormpath.normline_pt(-91.8845, -30.1806, -92.0183, -29.577),
    #ormpath.normline_pt(-92.0183, -29.577, -92.099, -28.9641),
    #ormpath.normline_pt(-92.099, -28.9641, -92.126, -28.3465),
    #ormpath.normline_pt(-92.126, -28.3465, -92.126, 157.804),
    #ormpath.normline_pt(-92.126, 157.804, -93.1688, 158.469),
    #ormpath.normline_pt(-93.1688, 158.469, -94.1498, 159.221),
    #ormpath.normline_pt(-94.1498, 159.221, -95.0613, 160.057),
    #ormpath.normline_pt(-95.0613, 160.057, -95.8967, 160.968),
    #ormpath.normline_pt(-95.8967, 160.968, -96.6494, 161.949),
    #ormpath.normline_pt(-96.6494, 161.949, -97.3138, 162.992),
    #ormpath.normline_pt(-97.3138, 162.992, -97.8847, 164.089),
    #ormpath.normline_pt(-97.8847, 164.089, -98.3578, 165.231),
    #ormpath.normline_pt(-98.3578, 165.231, -98.7297, 166.41),
    #ormpath.normline_pt(-98.7297, 166.41, -98.9973, 167.618),
    #ormpath.normline_pt(-98.9973, 167.618, -99.1587, 168.843),
    #ormpath.normline_pt(-99.1587, 168.843, -99.2126, 170.079),
    #ormpath.normline_pt(-99.2126, 170.079, -99.1587, 171.314),
    #ormpath.normline_pt(-99.1587, 171.314, -98.9973, 172.54),
    #ormpath.normline_pt(-98.9973, 172.54, -98.7297, 173.747),
    #ormpath.normline_pt(-98.7297, 173.747, -98.3578, 174.926),
    #ormpath.normline_pt(-98.3578, 174.926, -97.8847, 176.069),
    #ormpath.normline_pt(-97.8847, 176.069, -97.3138, 177.165),
    #ormpath.normline_pt(-97.3138, 177.165, -96.6494, 178.208),
    #ormpath.normline_pt(-96.6494, 178.208, -95.8967, 179.189),
    #ormpath.normline_pt(-95.8967, 179.189, -95.0613, 180.101),
    #ormpath.normline_pt(-95.0613, 180.101, -94.1498, 180.936),
    #ormpath.normline_pt(-94.1498, 180.936, -93.1688, 181.689),
    #ormpath.normline_pt(-93.1688, 181.689, -92.126, 182.353),
    #ormpath.normline_pt(-92.126, 182.353, -91.0292, 182.924),
    #ormpath.normline_pt(-91.0292, 182.924, -89.8869, 183.397),
    #ormpath.normline_pt(-89.8869, 183.397, -88.7077, 183.769),
    #ormpath.normline_pt(-88.7077, 183.769, -87.5005, 184.037),
    ], closed=1) # >>>
    p = normpath.normpath([nsp])
    c.stroke(p, [trafo.translate(*move)])
    hard_test(c, p, 0.47504345 - 5*normpath._epsilon, parallel(0.0), move, "Y", checklens=[[37,14],[37,14],[7,7],[7,7]])
    hard_test(c, p, 0.47504345 + 5*normpath._epsilon, parallel(0.0), move, "Y", checklens=[[37,7],[37,7],[7,7,7],[7,7,7]])
    hard_test(c, p, 0.5, parallel(0.0), move, "Y", checklens=[[36,7],[36,7],[6,5,6,3,7,3],[6,3,7,3,5,6]])
    hard_test(c, p, 0.6125, parallel(0.0), move, "Y", checklens=[[35],[35],[3],[3]])
    hard_test(c, p, 0.6130, parallel(0.0), move, "Y", checklens=[[35],[35],[],[]])

    # a path of two subpaths:
    # and with inner intersections to be cut away by _between_paths
    move = (0, 0)
    p = path.circle(-6, 0, 2)
    p += path.path(path.moveto(0,0), path.curveto(0,16, -11,5, 5,5))
    p += path.path(path.lineto(5,4), path.lineto(7,4), path.lineto(7,6), path.lineto(4,6),
                   path.lineto(4,7), path.lineto(5,7), path.lineto(3,1), path.closepath())
    p = p.transformed(trafo.scale(0.5)).normpath()
    hard_test(c, p, 0.05, parallel(0.0), move, "Z", checklens=[[10,2,8],[9,2,10],[13,4,4],[13,4,4]])
    hard_test(c, p, 0.3, parallel(0.0), move, "Z", checklens=[[10,4,5],[4,6,10],[13,4],[13,4]])
    hard_test(c, p, 0.6, parallel(0.0), move, "Z", checklens=[[],[],[13,4],[13,4]])


def testlinesmoothed(c):

    # dependence on turnangle
    p = path.path(path.moveto(0, 0), path.lineto(1, 0), path.lineto(2, 1), path.lineto(4, -1),
                  path.lineto(5, 0), path.lineto(6, 0))
    c.stroke(p)
    d = linesmoothed()
    c.stroke(p, [d])
    c.stroke(p, [d(atleast=True), color.rgb.red])
    c.stroke(p, [d(lcurl=None), color.rgb.green])
    c.stroke(p, [d(rcurl=None), color.rgb.blue])

    p.append(path.lineto(3, 3))
    p.append(path.closepath())

    c.stroke(p, [d, color.cmyk.Orange])

c=canvas.canvas()
dotest(c, 0, -12, "testparallel_1")
dotest(c, 6, 13, "testparallel_2")
dotest(c, 16, 12, "testlinesmoothed")
dotest(c, 16, 4, "testcycloid")
dotest(c, 20, -10, "testcornersmoothed")
c.writeEPSfile("test_deformer", page_paperformat=document.paperformat.A4, page_rotated=0, page_fittosize=1)
c.writePDFfile("test_deformer")
c.writeSVGfile("test_deformer")

# vim:foldmethod=marker:foldmarker=<<<,>>>



from pyx import *

a, b = 2, 9 # integral area

p = graph.axis.painter.regular(basepathattrs=[deco.earrow.normal],
                               titlepos=0.98, titledirection=None)
ticks = [graph.axis.tick.tick(a, label="$a$"),
         graph.axis.tick.tick(b, label="$b$")]
g = graph.graphxy(width=8, x2=None, y2=None,
                  x=graph.axis.linear(title="$x$", min=0, max=10,
                                      manualticks=ticks,
                                      parter=None, painter=p),
                  y=graph.axis.linear(title="$y$", parter=None, painter=p))
d = g.plot(graph.data.function("y(x)=(x-3)*(x-5)*(x-7)"))
g.finish()
p = d.path # the path is available after the graph is finished

pa = g.xgridpath(a)
pb = g.xgridpath(b)
(splita,), (splitpa,) = p.intersect(pa)
(splitb,), (splitpb,) = p.intersect(pb)
area = (pa.split([splitpa])[0] <<
        p.split([splita, splitb])[1] <<
        pb.split([splitpb])[0].reversed())
area[-1].close()
g.stroke(area, [deco.filled([color.gray(0.8)])])
g.text(g.pos(0.5 * (a + b), 0)[0], 1,
       r"\int_a^b f(x){\rm d}x", [text.halign.center, text.mathmode])
g.writeEPSfile()
g.writePDFfile()
g.writeSVGfile()



from pyx import *

text.set(mode="latex")
text.preamble(r"\usepackage{exscale}")

spameggswidth = 2.7
arrowdist = 0.1
arrowlen = 0.2
textdist = 0.1

c = canvas.canvas()


def label(x, y, width, t): # x == -1 -> left; x == 1 -> right
    c.stroke(path.line(width*(0.5*x+0.5) + x*arrowdist, y,
                       width*(0.5*x+0.5) + x*(arrowdist + arrowlen), y),
             [deco.barrow.Small])
    c.text(width*(0.5*x+0.5) + x*(arrowdist + arrowlen + textdist), y,
           t, [text.vshift.mathaxis, text.boxhalign(0.5-0.5*x)])


t = c.text(0, 0, r"\noindent spam \& eggs", [text.parbox(2.7), text.size.Huge])
t2 = text.text(0, 0, "eggs", [text.size.Huge])

c.stroke(t.bbox().path(), [style.linewidth.THin])
c.stroke(path.line(0, 0, 2.7, 0) +
         path.line(0, t2.depth-t.depth, 2.7, t2.depth-t.depth),
         [style.linestyle.dashed, style.linewidth.THin])

label(-1, t.height, spameggswidth, "valign.top")
label(-1, 0.5*(t.height-t.depth), spameggswidth, "valign.middle")
label(-1, -t.depth, spameggswidth, "valign.bottom")
label(1, 0, spameggswidth, "parbox.top")
label(1, 0.5*(t2.depth-t.depth), spameggswidth, "parbox.middle")
label(1, t2.depth-t.depth, spameggswidth, "parbox.bottom")

mathypos = -2.5
mathstr = r"\sum_{i=0}^{\infty}\frac{1}{i^2}=\frac{1}{6}\pi^2"

t = c.text(0, mathypos, mathstr, [text.mathmode, text.vshift.mathaxis, text.size.LARGE])
t2 = text.text(0, mathypos, mathstr, [text.mathmode, text.size.LARGE])

c.stroke(t.bbox().path(), [style.linewidth.THin])
c.stroke(path.line(0, mathypos, t.width, mathypos)+
         path.line(0, mathypos+t.height-t2.height, t.width, mathypos+t.height-t2.height),
         [style.linestyle.dashed, style.linewidth.THin])

label(-1, mathypos, t.width, "vshift.mathaxis")
label(1, mathypos+t.height-t2.height, t.width, "baseline")

c.writeEPSfile("valign")



#!/usr/bin/env python
import sys; sys.path[:0] = ["../.."]

from pyx import *

c = canvas.canvas()

c.stroke(path.line(10*unit.u_pt, 10*unit.u_pt, 40*unit.u_pt, 40*unit.u_pt)+
         path.line(10*unit.u_pt, 40*unit.u_pt, 40*unit.u_pt, 10*unit.u_pt))
#         path.rect(10*unit.u_pt, 10*unit.u_pt, 30*unit.u_pt, 30*unit.u_pt))

c.writeEPSfile("cross", page_bboxenlarge=0)

c = canvas.canvas()

c.stroke(path.line(0,4,6,4), [style.linestyle.dashed])
c.insert(epsfile.epsfile(0, 4, "cross.eps", align="bl"))
c.insert(epsfile.epsfile(2, 4, "cross.eps", align="cl"))
c.insert(epsfile.epsfile(4, 4, "cross.eps", align="tl"))

c.stroke(path.line(3,6,3,10), [style.linestyle.dashed])
c.insert(epsfile.epsfile(3, 6, "cross.eps", align="bl"))
c.insert(epsfile.epsfile(3, 8, "cross.eps", align="bc"))
c.insert(epsfile.epsfile(3, 10, "cross.eps", align="br"))

c.insert(epsfile.epsfile(1, -1, "cross.eps", scale=1))
c.insert(epsfile.epsfile(1, -1, "cross.eps", scale=2))

c.insert(epsfile.epsfile(5, -1, "cross.eps", scale=1, align="cc"))
c.insert(epsfile.epsfile(5, -1, "cross.eps", scale=2, align="cc"))

c.insert(epsfile.epsfile(9, -1, "cross.eps", scale=1, align="tr"))
c.insert(epsfile.epsfile(9, -1, "cross.eps", scale=2, align="tr"))

c.insert(epsfile.epsfile(1, -5, "cross.eps"))
c.insert(epsfile.epsfile(1, -5, "cross.eps", width=2))

c.insert(epsfile.epsfile(5, -5, "cross.eps", scale=1, align="cc"))
c.insert(epsfile.epsfile(5, -5, "cross.eps", width=2, align="cc"))

c.insert(epsfile.epsfile(9, -5, "cross.eps", scale=1, align="tr"))
c.insert(epsfile.epsfile(9, -5, "cross.eps", width=2, align="tr"))

c.insert(epsfile.epsfile(1, -9, "cross.eps"))
c.insert(epsfile.epsfile(1, -9, "cross.eps", height=1.5))

c.insert(epsfile.epsfile(5, -9, "cross.eps", scale=1, align="cc"))
c.insert(epsfile.epsfile(5, -9, "cross.eps", height=1.5, align="cc"))

c.insert(epsfile.epsfile(9, -9, "cross.eps", scale=1, align="tr"))
c.insert(epsfile.epsfile(9, -9, "cross.eps", height=1.5, align="tr"))

c.insert(epsfile.epsfile(1, -13, "cross.eps"))
c.insert(epsfile.epsfile(1, -13, "cross.eps", width=2, height=1.5))

c.insert(epsfile.epsfile(5, -13, "cross.eps", scale=1, align="cc"))
c.insert(epsfile.epsfile(5, -13, "cross.eps", width=2,height=1.5, align="cc"))

c.insert(epsfile.epsfile(9, -13, "cross.eps", scale=1, align="tr"))
c.insert(epsfile.epsfile(9, -13, "cross.eps", width=2, height=1.5, align="tr"))

c.writeEPSfile("test_epsfile")



import random
import sys; sys.path[:0] = ["../.."]
from pyx import *

d1 = graph.data.points([(1.5, 1, 0.3),
                      (3, 0.5, 0.5),
                      (4, 0.5, 0.2),
                      (None, 0.5, 0.8),
                      (6, None, 0.9),
                      (7, 0.5, None),
                      (10, 2.5, 0.8),
                      (13, 0.5, -0.2),
                      (14, 0.5, 0.4)], x=1, dx=2, y=3)
d2 = graph.data.data(d1, y="x", dy="dx", x="y", copy=0)
d3 = graph.data.points([(1, 0.3),
                      (2, 0.5),
                      (3, -0.2),
                      (4, 0.8),
                      (5, 0.5)], x=1, y=2)
d4 = graph.data.data(d3, y="x", x="y")

c = canvas.canvas()

def draw(d, xpos, ypos, autohistogramaxisindex=0, autohistogrampointpos=0, fillablesteps=0, **kwargs):
    g = c.insert(graph.graphxy(xpos, ypos, width=8, **kwargs))
    g.plot(d, [graph.style.histogram(fillable=1, steps=fillablesteps, lineattrs=[color.rgb.green, style.linewidth.THIck, deco.filled([color.rgb.blue])],
                                     autohistogramaxisindex=autohistogramaxisindex, autohistogrampointpos=autohistogrampointpos)])
    g.plot(d, [graph.style.histogram(steps=1, lineattrs=[color.rgb.red, style.linewidth.Thick],
                                     autohistogramaxisindex=autohistogramaxisindex, autohistogrampointpos=autohistogrampointpos)])
    g.plot(d, [graph.style.histogram(steps=0,
                                     autohistogramaxisindex=autohistogramaxisindex, autohistogrampointpos=autohistogrampointpos)])

draw(d1, 10, 0)
draw(d2, 0, 0)
draw(d3, 10, 7, autohistogramaxisindex=0, autohistogrampointpos=0)
draw(d4, 0, 7, autohistogramaxisindex=1, autohistogrampointpos=1)
draw(d1, 10, 14, x=graph.axis.lin(min=3, max=14), y=graph.axis.lin(min=-0.1, max=0.45))
draw(d2, 0, 14, y=graph.axis.lin(min=3, max=14), x=graph.axis.lin(min=-0.1, max=0.45))
draw(d1, 10, 21, fillablesteps=1, x=graph.axis.lin(min=3, max=14), y=graph.axis.lin(min=-0.1, max=0.45))
draw(d2, 0, 21, fillablesteps=1, y=graph.axis.lin(min=3, max=14), x=graph.axis.lin(min=-0.1, max=0.45))
c.writeEPSfile("test_histogram")
c.writePDFfile("test_histogram")
c.writeSVGfile("test_histogram")



#!/usr/bin/env python
import sys;sys.path.insert(0, "..")
from math import *
from pyx import *
# File for invalid parametrisations of Bezier curves
# Some examples

def Dx_m(Dy):
    return (-1-2*Dy-sqrt((1+2*Dy)**2+3))/3.0

def Dx_p(Dy):
    return (-1-2*Dy+sqrt((1+2*Dy)**2+3))/3.0

def acurve(Dx_fct, Dy, Dx=None):
    if Dx is None:
        Dx = Dx_fct(Dy)
    p = path.curve(0,0, 0,1, Dx,1+Dy, 1,0)
    c = canvas.canvas()
    c.stroke(p, [deco.shownormpath()])
    c.text(-0.2, -10*unit.x_pt, r"\noindent$\Delta x=%g $\par\noindent$\Delta y=%g$"%(Dx,Dy),
           [text.parbox(4), text.size.footnotesize])
    return c

dx = 3
dy = -3
cc = acurve(Dx_m, 0)
c = acurve(Dx_m, 1); cc.insert(c, [trafo.translate(dx, 0)])
c = acurve(Dx_m, -1); cc.insert(c, [trafo.translate(2*dx, 0)])
c = acurve(Dx_m, -2); cc.insert(c, [trafo.translate(0, dy)])
c = acurve(Dx_p, -1); cc.insert(c, [trafo.translate(dx, dy)])
c = acurve(Dx_p, -3); cc.insert(c, [trafo.translate(2*dx, dy)])
cc.writePDFfile()




# -*- coding: utf-8 -*-
#
# PyX documentation build configuration file, created by
# sphinx-quickstart on Thu May 19 17:32:57 2011.
#
# This file is execfile()d with the current directory set to its containing dir.
#
# Note that not all possible configuration values are present in this
# autogenerated file.
#
# All configuration values have a default; values that are commented out
# serve to show the default.

import sys, os
sys.path.insert(0, '../pyx')
sys.path.insert(0, '..')
import pyx.version

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#sys.path.insert(0, os.path.abspath('.'))

# -- General configuration -----------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
#needs_sphinx = '1.0'

# Add any Sphinx extension module names here, as strings. They can be extensions
# coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = ['sphinx.ext.mathjax', 'sphinx.ext.todo', 'sphinx.ext.autodoc', 'sphinx.ext.doctest', 'sphinx_selective_exclude.eager_only']

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix of source filenames.
source_suffix = '.rst'

# The encoding of source files.
#source_encoding = 'utf-8-sig'

# The master toctree document.
master_doc = 'manual'

# General information about the project.
project = 'PyX'
copyright = pyx.version.date.split('/')[0] + ', Jörg Lehmann, Michael Schindler, André Wobst'

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#
# The short X.Y version.
version = '.'.join(pyx.version.version.split('.')[:1])
# The full version, including alpha/beta/rc tags.
release = pyx.version.version

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#language = None

# There are two options for replacing |today|: either, you set today to some
# non-false value, then it is used:
today = pyx.version.date
# Else, today_fmt is used as the format for a strftime call.
#today_fmt = '%B %d, %Y'

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
exclude_patterns = ['_build']

# The reST default role (used for this markup: `text`) to use for all documents.
#default_role = None

# If true, '()' will be appended to :func: etc. cross-reference text.
#add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
#add_module_names = True

# If true, sectionauthor and moduleauthor directives will be shown in the
# output. They are ignored by default.
#show_authors = False

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# A list of ignored prefixes for module index sorting.
#modindex_common_prefix = []


# -- Options for HTML output ---------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
html_theme = 'pyx'

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
html_theme_options = {}

# Add any paths that contain custom themes here, relative to this directory.
html_theme_path = ['theme']

# The name for this set of Sphinx documents.  If None, it defaults to
# "<project> v<release> documentation".
html_title = 'PyX %s Manual' % release

# A shorter title for the navigation bar.  Default is the same as html_title.
#html_short_title = None

# The name of an image file (relative to this directory) to place at the top
# of the sidebar.
#html_logo = None

# The name of an image file (within the static path) to use as favicon of the
# docs.  This file should be a Windows icon file (.ico) being 16x16 or 32x32
# pixels large.
#html_favicon = None

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = []

# If not '', a 'Last updated on:' timestamp is inserted at every page bottom,
# using the given strftime format.
#html_last_updated_fmt = '%b %d, %Y'

# If true, SmartyPants will be used to convert quotes and dashes to
# typographically correct entities.
#html_use_smartypants = True

# Custom sidebar templates, maps document names to template names.
#html_sidebars = {}
html_sidebars = {
   '**': ['localtoc.html', 'sourcelink.html', 'searchbox.html']
}

# Additional templates that should be rendered to pages, maps page names to
# template names.
#html_additional_pages = {}

# If false, no module index is generated.
#html_domain_indices = True

# If false, no index is generated.
#html_use_index = True

# If true, the index is split into individual pages for each letter.
#html_split_index = False

# If true, links to the reST sources are added to the pages.
#html_show_sourcelink = True

# If true, "Created using Sphinx" is shown in the HTML footer. Default is True.
#html_show_sphinx = True

# If true, "(C) Copyright ..." is shown in the HTML footer. Default is True.
#html_show_copyright = True

# If true, an OpenSearch description file will be output, and all pages will
# contain a <link> tag referring to it.  The value of this option must be the
# base URL from which the finished HTML is served.
#html_use_opensearch = ''

# This is the file name suffix for HTML files (e.g. ".xhtml").
#html_file_suffix = None

# Output file base name for HTML help builder.
htmlhelp_basename = 'PyXdoc'

todo_include_todos = True


# -- Options for LaTeX output --------------------------------------------------

# The paper size ('letter' or 'a4').
latex_paper_size = 'a4'

# The font size ('10pt', '11pt' or '12pt').
#latex_font_size = '10pt'

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title, author, documentclass [howto/manual]).
latex_documents = [
  ('manual', 'manual.tex', 'PyX Manual',
   'Jörg Lehmann, Michael Schindler, André Wobst', 'manual'),
]

# The name of an image file (relative to this directory) to place at the top of
# the title page.
#latex_logo = None

# For "manual" documents, if this is true, then toplevel headings are parts,
# not chapters.
#latex_use_parts = False

# If true, show page references after internal links.
#latex_show_pagerefs = False

# If true, show URL addresses after external links.
#latex_show_urls = False

# Additional stuff for the LaTeX preamble.
latex_preamble = r'''
  \hypersetup{pdftitle={PyX Manual},
              pdfauthor={Joerg Lehmann <joerg@pyx-project.org>, Michael Schindler <m-schindler@users.sourceforge.net>, Andre Wobst <wobsta@pyx-project.org>},
              pdfsubject={PyX Manual},
              pdfkeywords={PyX, graphics, manual}}
'''

# Documents to append as an appendix to all manuals.
#latex_appendices = []

# If false, no module index is generated.
#latex_domain_indices = True


# -- Options for manual page output --------------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    ('manual', 'pyx', 'PyX Manual',
     ['Jörg Lehmann, Michael Schindler, André Wobst'], 1)
]

# -- Other options  ------------------------------------------------------------

mathjax_path = 'https://cdnjs.cloudflare.com/ajax/libs/mathjax/2.7.4/MathJax.js?config=default'
todo_include_todos = True
autoclass_content = 'both'
autodoc_member_order = 'bysource'


# -- unprocessed function signature extractor ----------------------------------
# This feature is not robust. It does not parse the code properly, removes
# comments and whitespace doubtfully, requires '):' without space for
# termination, etc.

import inspect

def function_signature_lines(lines):
    # extract the lines of the function definition removing comments
    # and stripping spaces
    for line in lines:
        code = line.split("#", 1)[0].strip()
        yield code
        if code.endswith("):"):
            break

def unprocessed_function_signature(app, what, name, obj, options, sig, retann):
    # extract the unprocessed signature from the source
    if what not in ["class", "method", "staticmethod", "function"]:
        return
    if what == "class":
        # get the constructor (code copied from autodoc)
        obj = getattr(obj, "__init__", None)
        if obj is None or obj is object.__init__ or not \
           (inspect.ismethod(obj) or inspect.isfunction(obj)):
            return
    elif hasattr(obj, '__wrapped__'):
        obj = obj.__wrapped__
    src, line = inspect.findsource(obj)
    code = " ".join(function_signature_lines(src[line:])).split("(", 1)[1][:-2]
    if code.startswith("self, "):
        code = code[6:]
    elif code == "self":
        code = ""
    return "({})".format(code), retann

def remove_default_constructor_docstring(app, what, name, obj, options, lines):
    # remove default constructor docstring, i.e. when no own constructor is defined
    for i, line in enumerate(lines):
        lines[i] = line.replace("x.__init__(...) initializes x; see help(type(x)) for signature", "")

def setup(app):
    app.connect('autodoc-process-signature', unprocessed_function_signature)
    app.connect('autodoc-process-docstring', remove_default_constructor_docstring)



from pyx import *

n = 3       # number of masses
r = 3.0     # system radius
rcyc = 0.3  # radius of cycloid
nl = 13     # number of cycloid half-loops
rc = 0.5    # radius of masses
eps = 0.03  # extra spacing for surrounding circles

c = canvas.canvas()
springcircle = path.circle(0, 0, r)
masspositions = [i*springcircle.arclen()/n
                 for i in range(n)]
for springsegment in springcircle.split(masspositions):
    c.stroke(springsegment,
             [deformer.cycloid(rcyc, nl),
              deformer.smoothed(radius=0.1)])
for x, y in springcircle.at(masspositions):
    c.fill(path.circle(x, y, rc))
c.stroke(path.circle(0, 0, r - rc - eps))
c.stroke(path.circle(0, 0, r + rc + eps))
c.writeEPSfile("springs")



#!/usr/bin/env python
import sys;sys.path.insert(0, "..")
from math import *
from pyx import *
# File for invalid parametrisations of Bezier curves
# Check a-posteriori the sign of Delta y for the square-root solution of Delta x

def Dx_m(y):
    return (-1-2*y-sqrt((1+2*y)**2+3))/3.0
def Dx_p(y):
    return (-1-2*y+sqrt((1+2*y)**2+3))/3.0

con = {"Dx_m":Dx_m, "Dx_p":Dx_p}

g = graph.graphxy(width=10,
    x=graph.axis.lin(),
    y=graph.axis.lin(min=-2, max=2),
    key=graph.key.key(pos="bl"),
)
g.plot(graph.data.function("x(y)=6*y*Dx_m(y)", title="minus-sol, lhs", context=con), [graph.style.line([color.rgb.red])])
g.plot(graph.data.function("x(y)=(1-3*Dx_m(y))*(1-y-sqrt(1+y+y*y))", title="minus-sol, minus-rhs", context=con), [graph.style.line([color.rgb.blue, style.linestyle.dashed])])
g.plot(graph.data.function("x(y)=6*y*Dx_p(y)", title="plus-sol, lhs", context=con), [graph.style.line([color.rgb.green])])
g.plot(graph.data.function("x(y)=(1-3*Dx_p(y))*(1-y+sqrt(1+y+y*y))", title="plus-sol, plus-rhs", context=con), [graph.style.line([style.linestyle.dashed])])
g.plot(graph.data.function("x(y)=(1-3*Dx_p(y))*(1-y-sqrt(1+y+y*y))", title="plus-sol, minus-rhs", context=con), [graph.style.line([style.linestyle.dotted])])
g.plot(graph.data.function("x(y)=(1-3*Dx_m(y))*(1-y+sqrt(1+y+y*y))", title="minus-sol, plus-rhs", context=con), [graph.style.line([style.linestyle.dotted])])
g.writePDFfile()




#!/usr/bin/env python
# -*- coding: ISO-8859-1 -*-
#
# Copyright (C) 2011 Michael Schindler <m-schindler@users.sourceforge.net>
#
# This file is part of PyX (https://pyx-project.org/).
#
# PyX is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# PyX is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyX; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
from math import atan2, pi, radians
import sys; sys.path[:0] = ["../.."]
from pyx import canvas, unit, deco, trafo, metapost
# TODO why must we change the epsilon before the import of _epsilon?
# does the import make a copy or a reference?
import pyx.metapost.path as mppath
mppath.set(epsilon=2.0e-5)
from pyx.metapost.path import *
from pyx import path
from pyx.metapost.path import _knot, _epsilon
from pyx.metapost.mp_path import mp_endpoint, mp_explicit, mp_given, mp_curl, mp_open, mp_make_choices
epsilon = _epsilon

# internal tests: check that the reimplementation of mp_make_choices is correct:
# tested with the output of a mpost binary (modified to output the knot parameters)

def curve1(): # <<<
    """The open curve of Fig.3 on page 6 of mpman.pdf
    draw z0..z1..z2..z3..z4"""
    p = _knot(0, 0, mp_endpoint, None, None, mp_curl, 1, 1)
    knots = p
    p.next = _knot(60, 40, mp_open, None, 1, mp_open, None, 1)
    p = p.next
    p.next = _knot(40, 90, mp_open, None, 1, mp_open, None, 1)
    p = p.next
    p.next = _knot(10, 70, mp_open, None, 1, mp_open, None, 1)
    p = p.next
    p.next = _knot(30, 50, mp_curl, 1.0, 1, mp_endpoint, None, None)
    p = p.next
    p.next = knots
    refpoints = [
      ((None, None), (0, 0), (26.764633, -1.8454285)),
      ((51.409393, 14.584412), (60, 40), (67.098755, 61.001877)),
      ((59.762527, 84.57518), (40, 90), (25.357147, 94.01947)),
      ((10.480637, 84.502197), (10, 70), (9.628952, 58.804214)),
      ((18.804214, 49.628952), (30, 50), (None, None))]
    return knots, refpoints
# >>>
def curve2(): # <<<
    """The closed curve of Fig.4a on page 6 of mpman.pdf
    draw z0...z1...z2...z3...z4...cycle"""
    p = _knot(0, 0, mp_open, None, 1, mp_open, None, 1)
    knots = p
    p.next = _knot(60, 40, mp_open, None, 1, mp_open, None, 1)
    p = p.next
    p.next = _knot(40, 90, mp_open, None, 1, mp_open, None, 1)
    p = p.next
    p.next = _knot(10, 70, mp_open, None, 1, mp_open, None, 1)
    p = p.next
    p.next = _knot(30, 50, mp_open, None, 1, mp_open, None, 1)
    p = p.next
    p.next = knots
    refpoints = [
      ((-4.105545, 21.238037), (0, 0), (5.187561, -26.835297)),
      ((60.360733, -18.40036), (60, 40), (59.877136, 59.889008)),
      ((57.338959, 81.642029), (40, 90), (22.399872, 98.483871)),
      ((4.7240448, 84.463684), (10, 70), (13.386368, 60.716507)),
      ((26.355911, 59.135101), (30, 50), (39.194092, 26.951981))]
    return knots, refpoints
# >>>
def curve3(): # <<<
    """The open curve of Fig.6 on page 8 of mpman.pdf
    draw z0..z1{up}..z2{left}..z3..z4"""
    p = _knot(0, 0, mp_endpoint, None, None, mp_curl, 1, 1)
    knots = p
    # XXX: given angle is copied over from one side to the other:
    # this is mimicked by the default values of roughknot
    p.next = _knot(60, 40, mp_given, 0.5*pi, 1, mp_given, 0.5*pi, 1)
    p = p.next
    p.next = _knot(40, 90, mp_given, pi, 1, mp_given, pi, 1)
    p = p.next
    p.next = _knot(10, 70, mp_open, None, 1, mp_open, None, 1)
    p = p.next
    p.next = _knot(30, 50, mp_curl, 1, 1, mp_endpoint, None, None)
    p = p.next
    p.next = knots
    refpoints = [
      ((None, None), (0, 0), (28.543137, -11.892975)),
      ((60, 9.0782776), (60, 40), (60, 63.263458)),
      ((60.129883, 90), (40, 90), (25.690277, 90)),
      ((11.52742, 83.230453), (10, 70), (8.666214, 58.446793)),
      ((18.446793, 48.666214), (30, 50), (None, None))]
    return knots, refpoints
# >>>
def curve4(a): # <<<
    """Some of the curves of Figs. 7 and 8 on page 8 of mpman.pdf
    beginfig(7)
    for a=-9 upto 7:
      draw (0,0){dir 45}..{dir 10a}(6cm,0);
    endfor
    endfig;"""
    if a == -90:
        refpoints = [((None, None), (0, 0), (72.980957, 72.980957)),
                     ((170.0787, 61.772141), (170.0787, 0), (None, None))]
    elif a == 0:
        refpoints = [((None, None), (0, 0), (44.36261, 44.36261)),
                     ((110.4153, 0), (170.0787, 0), (None, None))]
    elif a == 70:
        refpoints = [((None, None), (0, 0), (41.195404, 41.195404)),
                     ((138.80974, -85.909775), (170.0787, 0), (None, None))]
    else:
        refpoints = None

    p = _knot(0, 0, mp_endpoint, None, None, mp_given, 0.25*pi, 1)
    knots = p
    p.next = _knot(170.078740157, 0, mp_given, radians(a), 1, mp_endpoint, None, None)
    p = p.next
    p.next = knots
    return knots, refpoints
# >>>
def curve5(): # <<<
    """The right curve of Fig. 9 on page 9 of mpman.pdf
    draw z0{up}...z1{right}...z2{down}"""
    p = _knot(0, 0, mp_endpoint, None, None, mp_given, 0.5*pi, -1)
    knots = p
    p.next = _knot(100, 20, mp_given, 0, -1, mp_given, 0, -1)
    p = p.next
    p.next = _knot(200, 0, mp_given, -0.5*pi, -1, mp_endpoint, None, None)
    p = p.next
    p.next = knots

    refpoints = [
      ((None, None), (0, 0), (0, 19.995117)),
      ((56.625137, 20), (100, 20), (143.37486, 20)),
      ((200, 19.995117), (200, 0), (None, None))]
    return knots, refpoints
# >>>
def curve6a(): # <<<
    """The first curve of Fig.10 on page 9 of mpman.pdf
    draw z0..z1..tension 1 and 1..z2..z3;"""
    p = _knot(0, 0, mp_endpoint, None, None, mp_curl, 1, 1)
    knots = p
    p.next = _knot(50, 50, mp_open, None, 1, mp_open, None, 1)
    p = p.next
    p.next = _knot(150, 50, mp_open, None, 1, mp_open, None, 1)
    p = p.next
    p.next = _knot(200, 0, mp_curl, 1, 1, mp_endpoint, None, None)
    p = p.next
    p.next = knots

    refpoints = [
      ((None, None), (0, 0), (10.747421, 21.688171)),
      ((28.311829, 39.252579), (50, 50), (81.50528, 65.612213)),
      ((118.49472, 65.612213), (150, 50), (171.68817, 39.252579)),
      ((189.25258, 21.688171), (200, 0), (None, None))]
    return knots, refpoints
# >>>
def curve6b(): # <<<
    """The first curve of Fig.10 on page 9 of mpman.pdf
    draw z0..z1..tension 1.3 and 1.3..z2..z3;"""
    p = _knot(0, 0, mp_endpoint, None, None, mp_curl, 1, 1)
    knots = p
    p.next = _knot(50, 50, mp_open, None, 1, mp_open, None, 1.3)
    p = p.next
    p.next = _knot(150, 50, mp_open, None, 1.3, mp_open, None, 1)
    p = p.next
    p.next = _knot(200, 0, mp_curl, 1, 1, mp_endpoint, None, None)
    p = p.next
    p.next = knots

    refpoints = [
      ((None, None), (0, 0), (7.0810547, 24.08403)),
      ((25.91597, 42.918945), (50, 50), (75.109573, 57.382568)),
      ((124.89043, 57.382568), (150, 50), (174.08403, 42.918945)),
      ((192.91895, 24.08403), (200, 0), (None, None))]
    return knots, refpoints
# >>>
def curve6c(): # <<<
    """The first curve of Fig.10 on page 9 of mpman.pdf
    draw z0..z1..tension 2.5 and 1..z2..z3;"""
    p = _knot(0, 0, mp_endpoint, None, None, mp_curl, 1, 1)
    knots = p
    p.next = _knot(50, 50, mp_open, None, 1, mp_open, None, 2.5)
    p = p.next
    p.next = _knot(150, 50, mp_open, None, 1, mp_open, None, 1)
    p = p.next
    p.next = _knot(200, 0, mp_curl, 1, 1, mp_endpoint, None, None)
    p = p.next
    p.next = knots

    refpoints = [
      ((None, None), (0, 0), (5.4299469, 25.023956)),
      ((24.976044, 44.570053), (50, 50), (63.245346, 52.8741)),
      ((117.57947, 59.957458), (150, 50), (173.92447, 42.651978)),
      ((192.65198, 23.924469), (200, 0), (None, None))]
    return knots, refpoints
# >>>
def curve7(): # <<<
    """The first curve of Fig.10 on page 9 of mpman.pdf
    draw z0..z1..tension atleast 1..{curl 2}z2..z3{-1,-2}..tension 3 and 4..z4..controls z45 and z54..z5;"""

    p = _knot(0, 0, mp_endpoint, None, None, mp_curl, 1, 1) # z0
    knots = p
    p.next = _knot(50, 50, mp_open, None, 1, mp_open, None, -1) # z1
    p = p.next
    # XXX: curl value is copied over from one side to the other:
    # this is mimicked by the default values of roughknot
    p.next = _knot(80, 0, mp_curl, 2, -1, mp_curl, 2, 1) # z2
    p = p.next
    p.next = _knot(0, -20, mp_given, atan2(-2, -1), 1, mp_given, atan2(-2, -1), 3) # z3
    p = p.next
    p.next = _knot(50, -20, mp_open, None, 4, mp_explicit, -10, -50) # z4
    p = p.next
    p.next = _knot(150, 0, mp_explicit, 100, 50, mp_endpoint, None, None) # z5
    p = p.next
    p.next = knots

    refpoints = [
      ((None, None), (0, 0), (-2.2696381, 28.501495)),
      ((21.498505, 52.269638), (50, 50), (78.324738, 47.744446)),
      ((94.584061, 19.255417), (80, 0), (57.765503, 24.445801)),
      ((16.6745, 13.348999), (0, -20), (-11.100037, -42.200073)),
      ((80.924652, -4.537674), (50, -20), (-10, -50)),
      ((100, 50), (150, 0), (None, None))]
    return knots, refpoints
# >>>
def curve8a(): # <<<
    """Testing degenerate points
    draw (0,0)..(100,50)..(100,50)..{curl 1}(200,0)..(100,-50)..(100,-50)..(0,0)"""

    p = _knot(0.0, 0.0, mp_endpoint, None, None, mp_curl, 1.0, 1.0)
    knots = p
    p.next = _knot(100.0, 50.0, mp_open, None, 1, mp_open, None, 1.0)
    p = p.next
    p.next = _knot(100.0, 50.0, mp_open, None, 1, mp_open, None, 1.0)
    p = p.next
    p.next = _knot(200.0, 0.0, mp_curl, 1, 1, mp_curl, 1, 1)
    p = p.next
    p.next = _knot(100.0, -50.0, mp_open, None, 1, mp_open, None, 1.0)
    p = p.next
    p.next = _knot(100.0, -50.0, mp_open, None, 1, mp_open, None, 1.0)
    p = p.next
    p.next = _knot(0.0, 0.0, mp_curl, 1, 1, mp_endpoint, None, None)
    p = p.next
    p.next = knots

    refpoints = [
      ((None, None), (0, 0), (33.333328, 16.666672)),
      ((66.666672, 33.333328), (100, 50), (100, 50)),
      ((100, 50), (100, 50), (133.33333, 33.333328)),
      ((166.66667, 16.666672), (200, 0), (166.66667, -16.666672)),
      ((133.33333, -33.333328), (100, -50), (100, -50)),
      ((100, -50), (100, -50), (66.666672, -33.333328)),
      ((33.333328, -16.666672), (0, 0), (None, None))]
    return knots, refpoints
# >>>
def curve8b(): # <<<
    """Testing degenerate points
    draw (0,0)..tension 2 and 3..(100,50)..tension 3 and 4..(100,50)..tension 4 and 2..{curl 1}(200,0)..tension 2 and 3..(100,-50)..tension 3 and 4..(100,-50)..tension 4 and 2..(0,0)"""

    p = _knot(0.0, 0.0, mp_endpoint, None, None, mp_curl, 1, 2)
    knots = p
    p.next = _knot(100.0, 50.0, mp_open, None, 3, mp_open, None, 3)
    p = p.next
    p.next = _knot(100.0, 50.0, mp_open, None, 4, mp_open, None, 4)
    p = p.next
    p.next = _knot(200.0, 0.0, mp_curl, 1, 2, mp_curl, 1, 2)
    p = p.next
    p.next = _knot(100.0, -50.0, mp_open, None, 3, mp_open, None, 3)
    p = p.next
    p.next = _knot(100.0, -50.0, mp_open, None, 4, mp_open, None, 4)
    p = p.next
    p.next = _knot(0.0, 0.0, mp_curl, 1, 2, mp_endpoint, None, None)
    p = p.next
    p.next = knots

    refpoints = [
      ((None, None), (0, 0), (16.666672, 8.3333282)),
      ((88.888885, 44.444443), (100, 50), (100, 50)),
      ((100, 50), (100, 50), (108.33333, 45.833328)),
      ((183.33333, 8.3333282), (200, 0), (183.33333, -8.3333282)),
      ((111.11111, -44.444443), (100, -50), (100, -50)),
      ((100, -50), (100, -50), (91.666672, -45.833328)),
      ((16.666672, -8.3333282), (0, 0), (None, None))]
    return knots, refpoints
# >>>
def curve9(): # <<<
    """Testing all parts of the code.
    This cannot be tested on the command level."""

    p = _knot(0, 0, mp_endpoint, None, None, mp_curl, 1, 1)
    knots = p
    p.next = _knot(50, 10, mp_open, None, 1, mp_explicit, 50, 10)
    p = p.next
    p.next = _knot(100, 0, mp_explicit, 100, 0, mp_open, None, 1)
    p = p.next
    p.next = _knot(150, 50, mp_explicit, 149, 49, mp_open, None, 1)
    p = p.next
    p.next = _knot(200.0, 0.0, mp_curl, 1, 1, mp_endpoint, None, None)
    p = p.next
    p.next = knots

    refpoints = [
      ((None, None), (0, 0), (16.666672, 3.3333282)),
      ((33.333328, 6.6666718), (50, 10), (50, 10)),
      ((100, 0), (100, 0), (116.66667, 16.666672)),
      ((149.65987, 49.659866), (150, 50), (183.33333, 83.333328)),
      ((233.33333, 33.333328), (200, 0), (None, None))]

    return knots, refpoints
# >>>
def curve10(): # <<<
    """Testing all parts of the code: test the "else" in item 364
    This cannot be tested on the command level."""
    p = _knot(0, 0, mp_open, None, -1, mp_open, None, 1) # z0
    knots = p
    # ltype is curl, and not both tensions are 1:
    # This is already corrected by the parser of metapost
    p.next = _knot(60, 40, mp_curl, 2, 2, mp_open, None, -1) # z1
    p = p.next
    p.next = _knot(40, 90, mp_open, None, -1, mp_open, None, -1) # z2
    p = p.next
    p.next = _knot(10, 70, mp_open, None, -1, mp_open, None, -1) # z3
    p = p.next
    p.next = _knot(30, 50, mp_open, None, -1, mp_open, None, -1) # z4
    p = p.next
    p.next = knots

    refpoints = [
      ((-10.430161, 23.006058), (0, 0), (23.677628, -52.226303)),
      ((79.551056, 19.028336), (60, 40), (53.081604, 57.296005)),
      ((40, 90), (40, 90), (22.69722, 100.22987)),
      ((3.934021, 85.090485), (10, 70), (13.668121, 60.874741)),
      ((26.782501, 59.383743), (30, 50), (37.508362, 28.102051))]
    return knots, refpoints
# >>>

def myprint(knots): # <<<
    str = repr(knots)
    p = knots.next
    while not p is knots:
        str += " "
        str += repr(p)
        p = p.next
    return str
# >>>
def mypath(knots): # <<<
    p = path.path(path.moveto_pt(knots.x_pt, knots.y_pt))
    cx, cy = knots.rx_pt, knots.ry_pt
    prev = knots
    k = knots.next
    while not k is knots:
        p.append(path.curveto_pt(cx, cy, k.lx_pt, k.ly_pt, k.x_pt, k.y_pt))
        cx, cy = k.rx_pt, k.ry_pt
        prev = k
        k = k.next
    if knots.ltype == mp_explicit:
        p.append(path.curveto_pt(prev.rx_pt, prev.ry_pt, knots.lx_pt, knots.ly_pt, knots.x_pt, knots.y_pt))
        p.append(path.closepath())
    return p
# >>>
def check(knots, refpoints, eps=1.0e-3, rel=1.0e-5): # <<<
    if refpoints is None:
        return
    assert knots.linked_len() == len(refpoints)
    p = knots
    for i, (left, coord, right) in enumerate(refpoints):
        if left[0] is not None:
            assert abs(left[0]-p.lx_pt) < rel*(abs(left[0])+eps)
            assert abs(left[1]-p.ly_pt) < rel*(abs(left[1])+eps)
        assert abs(coord[0]-p.x_pt) < rel*(abs(coord[0])+eps)
        assert abs(coord[1]-p.y_pt) < rel*(abs(coord[1])+eps)
        if right[0] is not None:
            assert abs(right[0]-p.rx_pt) < rel*(abs(right[0])+eps)
            assert abs(right[1]-p.ry_pt) < rel*(abs(right[1])+eps)
        p = p.next
# >>>
def checkone(knots, refpoints): # <<<
    print(myprint(knots))
    mp_make_choices(knots, epsilon)
    print(myprint(knots))

    c = canvas.canvas()
    c.stroke(mypath(knots), [deco.shownormpath(), deco.earrow.normal])
    c.writePDFfile(bboxenlarge=unit.t_cm)
    c.writeEPSfile(bboxenlarge=unit.t_cm)
    c.writeSVGfile(bboxenlarge=unit.t_cm)

    check(knots, refpoints)
# >>>
def checkall(): # <<<
    c = None
    for knots, refpoints in [curve1(), curve2(), curve3(),
                             curve4(-90), curve4(0), curve4(70), curve5(),
                             curve6a(), curve6b(), curve6c(), curve7(),
                             curve8a(), curve8b(), curve9(), curve10()]:
        #print myprint(knots)
        mp_make_choices(knots, epsilon)
        #print myprint(knots)

        cc = canvas.canvas()
        cc.stroke(mypath(knots), [deco.shownormpath(), deco.earrow.normal])
        if c is None:
            c = canvas.canvas()
            c.insert(cc)
        else:
            c.insert(cc, [trafo.translate(0, c.bbox().bottom() - cc.bbox().top()-0.5)])

        check(knots, refpoints)
    c.writePDFfile()
    c.writeEPSfile()
    c.writeSVGfile()
# >>>

# test of the user interface:

def interface(): # <<<
    c = None

    for p in [
      # ordinary open path:
      mppath.path([beginknot(0,0), curve(), knot(6,4), curve(), knot(4,9), curve(), knot(1,7), curve(), endknot(3,5)], epsilon),
      # path containing two open subpaths:
      mppath.path([beginknot(0,0), curve(), endknot(6,4), beginknot(4,9), curve(), knot(1,7), curve(), endknot(3,5)], epsilon),
      # closed path:
      mppath.path([knot(0,0), curve(), knot(6,4), curve(), knot(4,9), curve(), knot(1,7), curve(), knot(3,5), curve()], epsilon),
      # open path, but with endpoints in the middle:
      mppath.path([knot(0,0), curve(), knot(6,4), curve(), endknot(4,9), beginknot(1,7), curve(), knot(3,5), curve()], epsilon),
      # the same path in the right order
      mppath.path([beginknot(1,7), curve(), knot(3,5), curve(), knot(0,0), curve(), knot(6,4), curve(), endknot(4,9)], epsilon),
      # include a line
      mppath.path([knot(0,0), curve(), knot(6,4), curve(), roughknot(4,9), line(), roughknot(1,7), curve(), knot(3,5), curve()], epsilon),
      # XXX the endpoints have "open" at their other sides, not "curl" as in the open example above
      mppath.path([knot(0,0), curve(), knot(6,4), curve(), knot(4,9), line(), knot(1,7), curve(), knot(3,5), curve()], epsilon),
      mppath.path([knot(0,0), curve(), knot(6,4), line(), knot(3,5), curve()], epsilon),
      mppath.path([knot(0,0), curve(), knot(6,4), curve(), knot(3,5), curve()], epsilon),
      # TODO the internal mp_make_choices treats this as closed, but the last curve is not plotted:
      mppath.path([knot(0,0), curve(), knot(6,4), curve(), knot(4,9), line(), knot(1,7), curve(), knot(3,5)], epsilon),
      # include a line with given angles
      mppath.path([knot(0,0), curve(), knot(6,4), curve(), knot(4,9), line(keepangles=True), knot(1,7), curve(), knot(3,5), curve()], epsilon),
      # include rough knots
      mppath.path([beginknot(0,0), curve(), roughknot(6,4,langle=90), curve(), roughknot(4,9,langle=-90),
            line(keepangles=True), roughknot(1,7,lcurl=3), curve(), endknot(3,5,angle=0)], epsilon),
    ]:
        cc = canvas.canvas()
        cc.stroke(p, [deco.shownormpath(), deco.earrow.normal])
        if c is None:
            c = cc
        else:
            c.insert(cc, [trafo.translate(c.bbox().right() - cc.bbox().left() + 0.5, 0)])
    c.writePDFfile()
    c.writeEPSfile()
    c.writeSVGfile()
# >>>


if __name__ == "__main__":
    #checkone(*curve10())
    checkall()
    interface()

# vim:foldmethod=marker:foldmarker=<<<,>>>
